from loguru import logger
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.exception import UnknownTarget
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.mirai import BotJoinGroupEvent
from graia.saya.builtins.broadcast.schema import ListenerSchema

from ...core.bot_config import BotConfig
from ...core.group_config import GroupPermission

channel = Channel.current()


@channel.use(ListenerSchema(listening_events=[BotJoinGroupEvent]))
async def main(app: Ariadne, joingroup: BotJoinGroupEvent):
    """
    收到入群事件
    """
    group = joingroup.group
    gp = GroupPermission(group)
    msg = [f"收到入群事件：{joingroup.group.name}（{joingroup.group.id}）"]
    if joingroup.inviter:
        msg.append(f"邀请者：{joingroup.inviter.name}（{joingroup.inviter.id}）")
    if gp.can_join():
        for admin in BotConfig.admins:
            try:
                await app.send_friend_message(
                    admin,
                    MessageChain(
                        msg,
                        "已自动加入",
                    ),
                )
            except UnknownTarget:
                logger.warning(f"由于未添加 {admin} 为好友，无法发送通知")
    else:
        await app.quit_group(group)
        for admin in BotConfig.admins:
            try:
                await app.send_friend_message(
                    admin,
                    MessageChain(
                        msg,
                        "该群不在白名单中，已自动退出",
                    ),
                )
            except UnknownTarget:
                logger.warning(f"由于未添加 {admin} 为好友，无法发送通知")



from fastapi import APIRouter, Depends, HTTPException, status

from .auth import verify_token
from ....core.control import Permission
from ....core.bot_config import BotConfig
from ....core.bot_config import _BotConfig
from ....model.fastapi import Info, ConfigResponse


router = APIRouter(tags=["Config"])


@router.get("/", summary="获取 Bot 当前配置", response_model=ConfigResponse)
async def get_config(info: Info = Depends(verify_token)):
    if info.permission < Permission.MASTER:
        raise HTTPException(status_code=status.HTTP_403_FORBIDDEN, detail="Permission denied")
    return ConfigResponse(code=0, message="success", data=BotConfig)


@router.post("/", summary="修改 Bot 配置", response_model=ConfigResponse)
async def edit_config(config: _BotConfig, info: Info = Depends(verify_token)):
    if info.permission < Permission.MASTER:
        raise HTTPException(status_code=status.HTTP_403_FORBIDDEN, detail="Permission denied")
    try:
        BotConfig.parse_obj(config)
        BotConfig.save()
        return ConfigResponse(code=0, message="success", data=BotConfig)
    except ValueError as e:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST, detail=_BotConfig.valueerror_parser(e)
        ) from e



from graia.ariadne.app import Ariadne
from graia.ariadne.event.message import GroupMessage
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.message.element import At
from graia.ariadne.message.parser.twilight import (
    ElementMatch,
    ElementResult,
    RegexMatch,
    Twilight,
)
from graia.ariadne.model import Group
from graia.saya import Channel
from graia.saya.builtins.broadcast.schema import ListenerSchema
import time
from ....core import BOT_Status
from ....core.bot_config import BotConfig
from ....core.control import Interval, Permission
from ....core.data import get_sub_by_group
from ....utils.time_tools import calc_time_total

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[
            Twilight(["at" @ ElementMatch(At, optional=True), RegexMatch(r"查看(本群)?(订阅|关注)列表")])
        ],
        decorators=[Permission.require(), Interval.require()],
    ),
)
async def sub_list(app: Ariadne, group: Group, at: ElementResult):
    if at.result:
        at_element: At = at.result  # type: ignore
        if at_element.target != BotConfig.Mirai.account:
            return
    sublist = get_sub_by_group(group.id)

    sublist_count = len(sublist)
    if sublist_count == 0:
        await app.send_group_message(group, MessageChain("本群未订阅任何 UP"))
    else:
        msg = [f"本群共订阅 {sublist_count} 个 UP\n注：带*号的表示该 UP 已被设定自定义昵称"]
        for i, sub in enumerate(sublist, 1):
            live_time = BOT_Status.living.get(str(sub.uid), -1)
            if live_time > 100000:
                live = f" 🔴直播中: {calc_time_total(time.time() - live_time)}"
            elif live_time >= 0:
                live = " 🔴直播中"
            else:
                live = ""
            msg.append(f"\n{i}. {f'*{sub.nick}' if sub.nick else sub.uname}（{sub.uid}）{live}")

        await app.send_group_message(group, MessageChain(msg))



from datetime import datetime, timedelta
from graia.ariadne.app import Ariadne
from fastapi import APIRouter, Depends, HTTPException, status

from .auth import verify_token
from ....core.control import Permission
from ....core.data import get_all_uid, get_push_count, get_talk_count
from ....model.fastapi import Info, HomeResponse, TalkCount, HomeItem



router = APIRouter(tags=["Home"])


@router.get("/all", summary="获取服务器基本信息", response_model=HomeResponse)
async def home(info: Info = Depends(verify_token)):
    if info.permission < Permission.MASTER:
        raise HTTPException(status_code=status.HTTP_403_FORBIDDEN, detail="Permission denied")
    app = Ariadne.current()
    return HomeResponse(
        code=0,
        message="success",
        data=HomeItem(
            group_count=len(await app.get_group_list()),
            friend_count=len(await app.get_friend_list()),
            uid_count=len(get_all_uid()),
            talk_count=[
                TalkCount(time=x["time"], count=x["count"])
                for x in get_talk_count(
                    datetime.now().replace(minute=0, second=0, microsecond=0)
                    - timedelta(days=1),
                    datetime.now().replace(minute=0, second=0, microsecond=0)
                    + timedelta(hours=1),
                )
            ],
            push_count=get_push_count(
                datetime.now().replace(hour=0, minute=0, second=0, microsecond=0),
                datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)
                + timedelta(days=1),
            ),
            all_push_count=get_push_count(),
        ),
    )



import asyncio

from creart import it
from typing import Union
from graia.saya import Channel, Saya
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Group, Member
from graia.broadcast.interrupt.waiter import Waiter
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import GroupMessage
from graia.broadcast.interrupt import InterruptControl
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import Twilight, RegexMatch

from ...core.control import Interval, Permission
from ...utils.strings import generate_verify_code

saya = Saya.current()
channel = Channel.current()
inc = it(InterruptControl)


class ConfirmWaiter(Waiter.create([GroupMessage], block_propagation=True)):
    def __init__(self, group: Union[Group, int], member: Union[Member, int], verify: str):
        self.group = group if isinstance(group, int) else group.id
        self.member = member if isinstance(member, int) else member.id
        self.verify = verify

    async def detected_event(
        self, app: Ariadne, group: Group, member: Member, message: MessageChain
    ):
        if self.group == group.id and self.member == member.id:
            msg = message.display
            if msg == self.verify:
                return True
            elif msg == "/quit cancel":
                return False
            else:
                await app.send_group_message(
                    self.group, MessageChain("请输入正确的验证码，或者发送 /quit cancel 来取消")
                )


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[Twilight(RegexMatch(r"([/.。?？!！])quit"))],
        decorators=[Permission.require(Permission.GROUP_ADMIN), Interval.require(20)],
    )
)
async def main(app: Ariadne, group: Group, member: Member):
    verify_code = generate_verify_code()
    await app.send_group_message(group, MessageChain(f"正在请求退出本群，请在30秒内输入验证码 {verify_code}"))
    try:
        res = await inc.wait(
            ConfirmWaiter(group, member, verify_code),
            timeout=30,
        )
    except asyncio.TimeoutError:
        res = False

    if res:
        await app.send_group_message(group, MessageChain("正在退出"))
        await asyncio.sleep(3)
        await app.quit_group(group)
    else:
        await app.send_group_message(group, MessageChain("已取消退群"))



from loguru import logger
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Group
from graia.ariadne.exception import UnknownTarget
from graia.ariadne.event.mirai import BotMuteEvent
from graia.ariadne.message.chain import MessageChain
from graia.saya.builtins.broadcast.schema import ListenerSchema

from ...core.bot_config import BotConfig
from ...core.group_config import GroupPermission

channel = Channel.current()


@channel.use(ListenerSchema(listening_events=[BotMuteEvent]))
async def main(app: Ariadne, group: Group, mute: BotMuteEvent):
    GroupPermission(group).remove_from_whitelist()
    if BotConfig.Event.mute:
        for admin in BotConfig.admins:
            try:
                await app.send_friend_message(
                    admin,
                    MessageChain(
                        "收到禁言事件，已退出该群",
                        f"\n群号：{group.id}",
                        f"\n群名：{group.name}",
                        f"\n操作者：{mute.operator.name} | {mute.operator.id}",
                    ),
                )
            except UnknownTarget:
                logger.warning(f"由于未添加 {admin} 为好友，无法发送通知")

    await app.quit_group(group)



import sys
import httpx
import asyncio

from yarl import URL
from io import BytesIO
from pathlib import Path
from loguru import logger
from zipfile import ZipFile
from rich.progress import (
    Progress,
    BarColumn,
    DownloadColumn,
    TimeElapsedColumn,
    TransferSpeedColumn,
)

from ..core.bot_config import BotConfig


DEFUALT_DYNAMIC_FONT = "HarmonyOS_Sans_SC_Medium.ttf"


font_path = Path("data", "font")
font_mime_map = {
    "collection": "font/collection",
    "otf": "font/otf",
    "sfnt": "font/sfnt",
    "ttf": "font/ttf",
    "woff": "font/woff",
    "woff2": "font/woff2",
}
font_path.mkdir(parents=True, exist_ok=True)


async def get_font(font: str = DEFUALT_DYNAMIC_FONT):
    logger.debug(f"font: {font}")
    url = URL(font)
    if url.is_absolute():
        if font_path.joinpath(url.name).exists():
            logger.debug(f"Font {url.name} found in local")
            return font_path.joinpath(url.name)
        else:
            logger.warning(f"字体 {font} 不存在，尝试从网络获取")
            async with httpx.AsyncClient() as client:
                resp = await client.get(font)
                if resp.status_code != 200:
                    raise ConnectionError(f"字体 {font} 获取失败")
                font_path.joinpath(url.name).write_bytes(resp.content)
                return font_path.joinpath(url.name)
    else:
        if not font_path.joinpath(font).exists():
            raise FileNotFoundError(f"字体 {font} 不存在")
        logger.debug(f"Font {font} found in local")
        return font_path.joinpath(font)


def get_font_sync(font: str = DEFUALT_DYNAMIC_FONT):
    return asyncio.run(get_font(font))


def font_init():
    # sourcery skip: extract-method
    font_url = (
        "https://mirrors.bfsu.edu.cn/pypi/web/packages/ad/97/"
        "03cd0a15291c6c193260d97586c4adf37a7277d8ae4507d68566c5757a6a/"
        "bbot_fonts-0.1.1-py3-none-any.whl"
    )
    lock_file = Path("data", "font", ".lock")
    lock_file.touch(exist_ok=True)
    if lock_file.read_text() != font_url:
        font_file = BytesIO()
        with Progress(
            "{task.description}",
            BarColumn(),
            "[progress.percentage]{task.percentage:>3.0f}%",
            "•",
            DownloadColumn(),
            "•",
            TransferSpeedColumn(),
            "•",
            TimeElapsedColumn(),
        ) as progress_bar:
            task = progress_bar.add_task("下载字体文件中", start=False)
            with httpx.stream("GET", font_url) as r:
                content_length = int(r.headers["Content-Length"])
                progress = 0
                progress_bar.update(task, total=content_length)
                progress_bar.update(task, completed=0)
                for chunk in r.iter_bytes():
                    font_file.write(chunk)
                    progress += len(chunk)
                    percent = progress / content_length * 100
                    progress_bar.advance(task, advance=percent)
                progress_bar.update(task, completed=content_length)
        with ZipFile(font_file) as z:
            fonts = [i for i in z.filelist if str(i.filename).startswith("bbot_fonts/font/")]
            for font in fonts:
                file_name = Path(font.filename).name
                local_file = font_path.joinpath(file_name)
                if not local_file.exists():
                    logger.info(local_file)
                    local_file.write_bytes(z.read(font))

        lock_file.write_text(font_url)
    else:
        logger.info("字体文件已存在，跳过下载")

    if BotConfig.Bilibili.dynamic_font:
        if BotConfig.Bilibili.dynamic_font_source == "remote":
            custom_font = URL(BotConfig.Bilibili.dynamic_font)
            if custom_font.is_absolute():
                if custom_font.name != URL(DEFUALT_DYNAMIC_FONT).name:
                    logger.info(get_font_sync(BotConfig.Bilibili.dynamic_font))
            else:
                logger.error("你输入的自定义字体不是一个有效的 URL，请检查！")
                sys.exit()
        else:
            custom_font = font_path.joinpath(BotConfig.Bilibili.dynamic_font)
            if not custom_font.exists():
                logger.error("你输入的自定义字体不存在（data/font），请检查！")
                sys.exit()



from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Group
from sentry_sdk import capture_exception
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import GroupMessage
from graia.ariadne.message.element import Image, Source
from bilireq.grpc.dynamic import grpc_get_user_dynamics
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    FullMatch,
    RegexMatch,
    ParamMatch,
    RegexResult,
)

from ...utils.uid_extract import uid_extract
from ...core.control import Interval, Permission
from ...utils.bilibili_request import get_b23_url
from ...utils.dynamic_shot import get_dynamic_screenshot

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[
            Twilight(
                [
                    FullMatch("查看动态"),
                    "up" @ ParamMatch(),
                    "offset" @ RegexMatch(r"\d+", optional=True),
                ]
            )
        ],
        decorators=[Permission.require(), Interval.require()],
    )
)
async def main(
    app: Ariadne,
    group: Group,
    source: Source,
    up: RegexResult,
    offset: RegexResult,
):
    uid = await uid_extract(text=up.result.display, groupid=group.id, show_error=True)
    if not uid or not uid.isdigit():
        return await app.send_group_message(
            group,
            MessageChain(uid or "未找到该 UP，请输入正确的 UP 群内昵称、UP 名、UP UID或 UP 首页链接"),
            quote=source,
        )

    if int(uid) == 0:
        return await app.send_group_message(group, MessageChain("UP 主不存在"), quote=source)

    try:
        res = await grpc_get_user_dynamics(int(uid))
    except Exception as e:
        capture_exception(e)
        return await app.send_group_message(group, MessageChain(f"获取动态失败：{e}"), quote=source)

    offset_num = int(offset.result.display) - 1 if offset.result else 0

    if res.list:
        if len(res.list) > 1:
            try:
                if res.list[0].modules[0].module_author.is_top:
                    dyn = res.list[1 + offset_num]
                else:
                    dyn = res.list[0 + offset_num]
            except IndexError:
                return await app.send_group_message(
                    group, MessageChain("你输入的数字过大，该 UP 的最后一页动态没有这么多条"), quote=source
                )
        else:
            dyn = res.list[0]
        shot_image = await get_dynamic_screenshot(dyn)
        return await app.send_group_message(
            group,
            MessageChain(
                Image(data_bytes=shot_image),
                "\n",
                await get_b23_url(f"https://t.bilibili.com/{dyn.extend.dyn_id_str}"),
            ),
            quote=source,
        )
    await app.send_group_message(group, MessageChain("该 UP 未发布任何动态"), quote=source)



import httpx
import click

from yarl import URL
from json import JSONDecodeError


def verify_mirai(host: str, account: int, verify_key: str):
    try:
        if not URL(host).is_absolute():
            return click.secho("Mirai HTTP API 地址不合法", fg="bright_red", bold=True)
    except Exception:
        return click.secho("Mirai HTTP API 地址不合法", fg="bright_red", bold=True)

    try:
        if httpx.get(f"{host}/about").json()["data"]["version"] < "2.6.1":
            return click.secho(
                "Mirai HTTP API 版本低于 2.6.1，可能会导致部分功能无法使用！请升级至最新版", fg="bright_red", bold=True
            )

        verify = httpx.post(f"{host}/verify", json={"verifyKey": verify_key}).json()
        if verify["code"] != 0:
            return click.secho("Mirai HTTP API 的 verifyKey 错误！", fg="bright_red", bold=True)

        bind = httpx.post(
            f"{host}/bind", json={"sessionKey": verify["session"], "qq": account}
        ).json()
        if bind["code"] != 0:
            return click.secho(
                f"Mirai HTTP API 验证错误：{bind['msg']}！", fg="bright_red", bold=True
            )
    except httpx.HTTPError as e:
        return click.secho(f"无法连接到 Mirai HTTP API，请检查地址是否正确！\n{e}", fg="bright_red", bold=True)
    except JSONDecodeError:
        return click.secho("输入的地址不为 Mirai HTTP API 的地址！", fg="bright_red", bold=True)

    if (
        httpx.post(
            f"{host}/release", json={"sessionKey": verify["session"], "qq": account}
        ).json()["code"]
        == 0
    ):
        return True



import json
import asyncio

from pathlib import Path
from loguru import logger
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from bilireq.login import refresh_token
from bilireq.utils import ResponseCodeError
from graia.scheduler.timers import crontabify
from graia.ariadne.message.chain import MessageChain
from graia.scheduler.saya.schema import SchedulerSchema

from ...core.bot_config import BotConfig
from ...core import BOT_Status, Bili_Auth, Status


channel = Channel.current()
login_cache_file = Path("data/login_cache.json")


@channel.use(SchedulerSchema(crontabify("0 0 * * *")))
async def main(app: Ariadne):
    logger.info("[BiliBili推送] 开始刷新 token")
    if BotConfig.Bilibili.username and BotConfig.Bilibili.password and Bili_Auth.tokens:
        BOT_Status.set_status(Status.INITIALIZED, False)

        while not BOT_Status.is_all_statuses_true(Status.LIVE_IDLE, Status.DYNAMIC_IDLE):
            await asyncio.sleep(0.1)

        try:
            resp = await refresh_token(auth=Bili_Auth)
            Bili_Auth.update(resp)
            logger.debug(await Bili_Auth.get_info())
            login_cache_file.write_text(json.dumps(dict(Bili_Auth), indent=2, ensure_ascii=False))
            logger.success(f"[BiliBili推送] 刷新 token 成功，token：{resp['token_info']}")
            await app.send_friend_message(
                BotConfig.master,
                MessageChain(f"[BiliBili推送] 刷新 token 成功\n{json.dumps(resp['token_info'], indent=2)}"),
            )
        except ResponseCodeError as e:
            logger.error(f"[BiliBili推送] 刷新 token 失败，{e}")
            await app.send_friend_message(
                BotConfig.master,
                MessageChain(f"[BiliBili推送] 刷新 token 失败，Bot 已关机，{e}"),
            )
            app.stop()
        BOT_Status.set_status(Status.INITIALIZED, True)
    else:
        logger.info("[BiliBili推送] 未启用登录，跳过 token 刷新")



import re
import contextlib

from loguru import logger

from .bilibili_request import hc


async def b23_extract(text: str):
    if "b23.tv" not in text and "b23.wtf" not in text:
        return None
    if not (b23 := re.compile(r"b23.(tv|wtf)[\\/]+(\w+)").search(text)):
        return None
    try:
        url = f"https://b23.tv/{b23[2]}"
        for _ in range(3):
            with contextlib.suppress(Exception):
                resp = await hc.get(url, follow_redirects=True)
                break
        else:
            return None
        url = resp.url
        logger.debug(f"b23.tv url: {url}")
        return str(url)
    except TypeError:
        return None



import json

from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Group
from graia.ariadne.message.element import Image
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import GroupMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import Twilight, FullMatch

from ...core import BOT_Status
from ...core.control import Permission
from ...utils.text2image import text2image

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[Twilight(FullMatch("/status"))],
        decorators=[Permission.require(Permission.MASTER)],
    )
)
async def main(app: Ariadne, group: Group):
    await app.send_group_message(
        group,
        MessageChain(
            Image(
                data_bytes=await text2image(
                    json.dumps(BOT_Status.to_dict(), indent=2, ensure_ascii=False)
                )
            )
        ),
    )



import re
from typing import Optional, Union

from loguru import logger
from pydantic import BaseModel, Extra

from ..core.data import get_sub_by_group
from .b23_extract import b23_extract
from .bilibili_request import search_user


class SearchUp(BaseModel, extra=Extra.ignore):
    title: str
    mid: int

    def __str__(self) -> str:
        return f"{self.title}({self.mid})"


class SearchResult(BaseModel, extra=Extra.ignore):
    items: list[SearchUp] = []


async def uid_extract(
    text: str, groupid: Optional[Union[int, str]] = None, show_error: bool = False
):
    logger.debug(f"[UID Extract] Original Text: {text}")
    if up_list := get_sub_by_group(groupid or 0):
        logger.debug(f"[UID Extract] Group {groupid} has {len(up_list)} Subscribers")
        if text.isdigit():
            logger.debug("[UID Extract] Text is a Number")
        for data in up_list:
            if text.isdigit():
                if data.uid == text:
                    logger.debug(
                        f"[UID Extract] Found UID from Group Subscribers: {data.uname}({data.uid})"
                    )
                    return str(data.uid)
            else:
                text_g = text.strip(""""'“”‘’""")
                if data.nick == text_g or data.uname == text_g:
                    logger.debug(
                        f"[UID Extract] Found UName from Group Subscribers: {data.uname}({data.uid})"
                    )
                    return str(data.uid)
        logger.debug("[UID Extract] No Subscriber found")
    b23_msg = await b23_extract(text) if "b23.tv" in text else None
    message = b23_msg or text
    logger.debug(f"[UID Extract] b23 extract: {message}")
    pattern = re.compile("^[0-9]*$|bilibili.com/([0-9]*)")
    if match := pattern.search(message):
        logger.debug(f"[UID Extract] Digit or Url: {match}")
        match = match[1] or match[0]
        return str(match)
    elif message.startswith("UID:"):
        pattern = re.compile("^\\d+")
        if match := pattern.search(message[4:]):
            logger.debug(f"[UID Extract] UID: {match}")
            return str(match[0])
    else:
        text_u = text.strip(""""'“”‘’""")
        if text_u != text:
            logger.debug(f"[UID Extract] Text is a Quoted Digit: {text_u}")
        logger.debug(f"[UID Extract] Searching UID in BiliBili: {text_u}")
        resp = await search_user(text_u)
        logger.debug(f"[UID Extract] Search result: {resp}")
        result = SearchResult(**resp)
        if result.items:
            for up in result.items:
                if up.title == text_u:
                    return str(up.mid)
                logger.debug("[UID Extract] No User found")
                return (
                    ("未找到该 UP，你可能在找：\n" + "\n".join([str(up) for up in result.items]))
                    if show_error
                    else None
                )
        logger.debug("[UID Extract] No User found")
        return "未找到该 UP，请输入正确的 UP 群内昵称、UP 名、UP UID或 UP 首页链接" if show_error else None



import sys
import json
import yaml
import click

from pathlib import Path
from typing import Optional, Literal
from pydantic import AnyHttpUrl, BaseModel, Extra, validator, ValidationError

DEFUALT_CONFIG_PATH = Path("data", "bot_config.yaml")


class _Mirai(BaseModel, extra=Extra.ignore):
    account: int
    verify_key: str = "mah"
    mirai_host: AnyHttpUrl = AnyHttpUrl("http://bbot-mah:6081", scheme="http")


class _Debug(BaseModel, extra=Extra.ignore):
    groups: Optional[list[int]] = [123]
    enable: bool = False

    # 规范 groups 内容
    @validator("groups")
    def specification_groups(cls, groups):
        if type(groups) == int:
            click.secho("groups 格式为 int, 已重置为 list[groups]", fg="bright_yellow")
            return [groups]
        elif type(groups) == list:
            return groups
        else:
            click.secho("debug.groups 为空或格式不为 list, 已重置为 None", fg="bright_yellow")

    # 验证是否可以开启 debug
    @validator("enable")
    def can_use_login(cls, enable, values):
        if not enable:
            return enable
        click.secho("已检测到开启 Debug 模式", fg="bright_yellow")
        try:
            if values["groups"]:
                return enable
            raise KeyError
        except KeyError:
            click.secho("已启用 Debug 但未填入合法的群号", fg="bright_red")
            sys.exit()


class _Bilibili(BaseModel, extra=Extra.ignore):
    username: Optional[str]
    password: Optional[str]
    use_login: bool = False
    use_browser: bool = True
    allow_fallback: bool = True
    mobile_style: bool = True
    render_style: Literal["bbot_default", "style_blue"] = "bbot_default"
    concurrency: int = 1
    dynamic_interval: int = 30
    dynamic_font: str = "HarmonyOS_Sans_SC_Medium.ttf"
    dynamic_font_source: Literal["local", "remote"] = "local"
    openai_summarization: bool = False
    openai_api_token: Optional[str] = None
    openai_model: str = "gpt-3.5-turbo-0301"
    openai_proxy: Optional[AnyHttpUrl] = None
    openai_cooldown: int = 60
    openai_whitelist_users: Optional[list[int]] = None
    openai_promot_version: int = 2
    use_wordcloud: bool = False
    use_bcut_asr: bool = False
    asr_length_threshold: int = 60
    captcha_address: Optional[AnyHttpUrl] = AnyHttpUrl("https://captcha-cd.ngworks.cn", scheme="https")
    content_resolve: bool = True

    # 验证是否可以登录
    @validator("use_login", always=True)
    def can_use_login(cls, use_login, values):
        if not use_login:
            return use_login
        click.secho("已检测到开启 BiliBili 登录模式，不推荐使用", fg="bright_yellow", bold=True)
        try:
            if isinstance(values["username"], int) and isinstance(values["password"], str):
                return use_login
        except KeyError:
            click.secho("已启用登录但未填入合法的用户名与密码", fg="bright_red")
            sys.exit()

    # 验证是否可以使用浏览器
    @validator("use_browser")
    def can_use_browser(cls, use_browser):
        if not use_browser:
            return use_browser
        click.secho("已检测到开启 BiliBili 浏览器模式", fg="bright_yellow")
        try:
            import playwright  # noqa  # type: ignore

            return use_browser
        except ImportError:
            click.secho("未安装 playwright，如需使用浏览器截图，请安装 graiax-playwright", fg="bright_red")
            sys.exit()

    # 验证是否可以使用 wordcloud
    @validator("use_wordcloud")
    def can_use_wordcloud(cls, use_wordcloud):
        if not use_wordcloud:
            return use_wordcloud
        click.secho("已检测到开启词云", fg="bright_yellow")
        try:
            import wordcloud  # noqa  # type: ignore
            import jieba  # noqa  # type: ignore

            return use_wordcloud
        except ImportError:
            click.secho("未安装 wordcloud，如需使用词云，请安装 wordcloud", fg="bright_red")
            sys.exit()

    # 验证是否可以启用 openai
    @validator("openai_api_token", always=True)
    def valid_openai_api(cls, openai_api_token: str, values: dict):
        if values.get("openai_summarization"):
            if not openai_api_token or not openai_api_token.startswith("sk-"):
                click.secho("openai_api_token 未填写的情况下无法开启 AI 视频总结", fg="bright_red")
                sys.exit()
        return openai_api_token

    # 验证 openai promt version
    @validator("openai_promot_version")
    def valid_openai_promot_version(cls, openai_promot_version):
        if openai_promot_version in [1, 2]:
            return openai_promot_version
        click.secho("openai_promot_version 只能为 1 或 2", fg="bright_red")
        sys.exit()

    # 验证 Bilibili gRPC 并发数
    @validator("concurrency")
    def limit_concurrency(cls, concurrency):
        if concurrency > 5:
            click.secho("gRPC 并发数超过 5，已自动调整为 5", fg="bright_yellow")
            return 50
        elif concurrency < 1:
            click.secho("gRPC 并发数小于 1，已自动调整为 1", fg="bright_yellow")
            return 1
        else:
            return concurrency

    @validator("render_style")
    def can_use_style(cls, render_style):
        if render_style == "bbot_default":
            return render_style
        try:
            import playwright  # noqa  # type: ignore

            return render_style
        except ImportError:
            click.secho("未安装 playwright，如需使用非默认的渲染样式，请安装 graiax-playwright", fg="bright_red")
            sys.exit()


class _Event(BaseModel, extra=Extra.ignore):
    mute: bool = True
    permchange: bool = True
    push: bool = True
    subscribe: bool = True


class _Webui(BaseModel, extra=Extra.ignore):
    webui_host: str = "0.0.0.0"
    webui_port: int = 6080
    webui_enable: bool = False


class _BotConfig(BaseModel, extra=Extra.ignore):
    Mirai: _Mirai
    Debug: _Debug
    Bilibili: _Bilibili
    Event: _Event
    Webui: _Webui
    log_level: str = "INFO"
    name: str = "BBot"
    master: int = 123
    admins: list[int] = []
    max_subsubscribe: int = 4
    access_control: bool = True
    update_check: bool = True
    use_richuru: bool = True

    # 验证 admins 列表
    @validator("admins")
    def verify_admins(cls, admins, values):
        if isinstance(admins, int):
            click.secho("admins 格式为 int, 已重置为 list[admins]", fg="bright_yellow")
            admins = [admins]
        elif not isinstance(admins, list) or not admins:
            if "master" not in values:
                raise ValueError("未查询到合法的 master")
            click.secho("admins 为空或格式不为 list, 已重置为 list[master]", fg="bright_yellow")
            return [values["master"]]
        try:
            if "master" not in values:
                return admins
            if values["master"] in admins:
                return admins
        except KeyError:
            click.secho("admins 内未包含 master 账号, 已自动添加", fg="bright_yellow")
            return admins.append(values["master"])

    # 读取配置文件
    @staticmethod
    def _read_file(file: Path = DEFUALT_CONFIG_PATH):
        bot_config: dict = yaml.load(file.read_bytes(), Loader=yaml.FullLoader)
        # 兼容旧配置, 将配置文件中的小写的配置项转为大写
        for old_config in ["mirai", "debug", "bilibili", "event"]:
            if old_config in bot_config:
                click.secho(
                    f"检测到旧版配置项, 转化为新版配置项: {old_config} => {old_config.capitalize()}",
                    fg="bright_yellow",
                )
                bot_config[old_config.capitalize()] = bot_config[old_config]
                del bot_config[old_config]
        return bot_config

    # ValueError解析
    @staticmethod
    def valueerror_parser(e: ValidationError):
        return {".".join([str(x) for x in err["loc"]]): err["msg"] for err in json.loads(e.json())}

    # 从配置文件中加载配置
    @classmethod
    def load(cls, file: Path = DEFUALT_CONFIG_PATH):
        # 如果文件不存在
        if not file.exists():
            raise FileNotFoundError
        return cls.parse_obj(cls._read_file())

    # 将配置保存至文件中
    def save(self, file: Path = DEFUALT_CONFIG_PATH):
        class NoAliasDumper(yaml.SafeDumper):
            def ignore_aliases(self, data):
                return True

        # 写入文件
        file.write_text(
            yaml.dump(json.loads(self.json()), Dumper=NoAliasDumper, sort_keys=False),
            encoding="utf-8",
        )



from bilireq.grpc.protos.bilibili.app.dynamic.v2.dynamic_pb2 import DynamicItem

from ..core.bot_config import BotConfig

from .pil_shot import pil_dynamic
from .detect_package import is_full


async def get_dynamic_screenshot(dyn: DynamicItem):
    if not is_full or not BotConfig.Bilibili.use_browser:
        return await pil_dynamic(dyn)

    from .browser_shot import browser_dynamic

    return (
        await browser_dynamic(dyn.extend.dyn_id_str) or await pil_dynamic(dyn)
        if BotConfig.Bilibili.allow_fallback
        else None
    )



from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Group
from graia.ariadne.message.element import At
from graia.ariadne.event.message import GroupMessage
from graia.ariadne.message.chain import MessageChain
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    RegexMatch,
    RegexResult,
    ElementMatch,
    ElementResult,
    WildcardMatch,
)


from ....utils.up_operation import unsubscribe_uid
from ....core.bot_config import BotConfig
from ....utils.uid_extract import uid_extract
from ....core.control import Interval, Permission

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[
            Twilight(
                [
                    "at" @ ElementMatch(At),
                    RegexMatch(r"(退订|取消?关注?)\s?(主播|[uU][pP])?"),
                    "anything" @ WildcardMatch(optional=True),
                ]
            )
        ],
        decorators=[Permission.require(Permission.GROUP_ADMIN), Interval.require()],
    )
)
async def main(app: Ariadne, group: Group, at: ElementResult, anything: RegexResult):

    at_result: At = at.result  # type: ignore
    if at_result.target == app.account:
        message = anything.result.display  # type: ignore

        uid = await uid_extract(message, group.id)

        if uid:
            msg = await unsubscribe_uid(uid, group.id)
            await app.send_friend_message(
                BotConfig.master,
                MessageChain(f"群 {group.name}（{group.id}）正在退订 UP：{uid}\n{msg}"),
            )
            if BotConfig.Event.subscribe:
                await app.send_group_message(
                    group,
                    MessageChain(msg),
                )
        else:
            await app.send_group_message(
                group,
                MessageChain("未找到该 UP，请输入正确的 UP 群内昵称、UP 名、UP UID或 UP 首页链接"),
            )



# coding=utf-8
"""
Xenon 管理 https://github.com/McZoo/Xenon/blob/master/lib/control.py
"""

import time

from asyncio import Lock
from typing import Optional
from collections import defaultdict
from typing import DefaultDict, Set, Tuple, Union
from graia.broadcast.exceptions import ExecutionStop
from graia.ariadne.event.message import GroupMessage
from graia.broadcast.builtin.decorators import Depend
from graia.ariadne.model import Member, MemberPerm, Friend

from .bot_config import BotConfig


class Permission:
    """
    用于管理权限的类，不应被实例化
    """

    MASTER = 30
    GROUP_ADMIN = 20
    USER = 10
    BANNED = 0
    DEFAULT = USER

    @classmethod
    def get(cls, member: Union[Member, int, str]) -> int:
        """
        获取用户的权限

        :param member: 用户实例或QQ号
        :return: 等级，整数
        """

        if isinstance(member, Member):
            user = member.id
            user_permission = member.permission
        elif isinstance(member, (int, str)):
            user = int(member)
            user_permission = None
        else:
            raise TypeError("member must be Member or int")

        if user == 80000000:  # 该号码为匿名 QQ 号
            raise ExecutionStop()

        if user in BotConfig.admins:
            return cls.MASTER
        elif user in []:
            return cls.BANNED
        elif user_permission in [MemberPerm.Administrator, MemberPerm.Owner]:
            return cls.GROUP_ADMIN
        else:
            return cls.DEFAULT

    @classmethod
    def require(cls, level: int = DEFAULT) -> Depend:
        """
        指示需要 `level` 以上等级才能触发，默认为至少 USER 权限

        :param level: 限制等级
        """

        def perm_check(member: Member):
            member_level = cls.get(member)

            if BotConfig.Debug.enable and member.group.id not in BotConfig.Debug.groups:
                raise ExecutionStop()
            if member_level >= level:
                return
            raise ExecutionStop()

        return Depend(perm_check)

    @classmethod
    def manual(cls, member: Union[Member, Friend], level: int = DEFAULT):
        member_level = cls.get(member.id)
        if (
            isinstance(member, Member)
            and BotConfig.Debug.enable
            and member.group.id not in BotConfig.Debug.groups
        ):
            raise ExecutionStop()
        if member_level >= level:
            return
        raise ExecutionStop()


class Interval:
    """
    用于冷却管理的类，不应被实例化
    """

    last_exec: DefaultDict[int, Tuple[int, float]] = defaultdict(lambda: (1, 0.0))
    sent_alert: Set[int] = set()
    lock: Optional[Lock] = None

    @classmethod
    async def get_lock(cls):
        if not cls.lock:
            cls.lock = Lock()
        return cls.lock

    @classmethod
    def require(
        cls,
        suspend_time: float = 10,
        max_exec: int = 1,
        override_level: int = Permission.MASTER,
    ):
        """
        指示用户每执行 `max_exec` 次后需要至少相隔 `suspend_time` 秒才能再次触发功能

        等级在 `override_level` 以上的可以无视限制

        :param suspend_time: 冷却时间
        :param max_exec: 在再次冷却前可使用次数
        :param override_level: 可超越限制的最小等级
        """

        async def cd_check(event: GroupMessage):
            if Permission.get(event.sender) >= override_level:
                return
            current = time.time()
            async with (await cls.get_lock()):
                last = cls.last_exec[event.sender.id]
                if current - cls.last_exec[event.sender.id][1] >= suspend_time:
                    cls.last_exec[event.sender.id] = (1, current)
                    if event.sender.id in cls.sent_alert:
                        cls.sent_alert.remove(event.sender.id)
                    return
                elif last[0] < max_exec:
                    cls.last_exec[event.sender.id] = (last[0] + 1, current)
                    if event.sender.id in cls.sent_alert:
                        cls.sent_alert.remove(event.sender.id)
                    return
                if event.sender.id not in cls.sent_alert:
                    cls.sent_alert.add(event.sender.id)
                raise ExecutionStop()

        return Depend(cd_check)

    @classmethod
    async def manual(
        cls,
        member: Union[Member, int],
        suspend_time: float = 10,
        max_exec: int = 1,
        override_level: int = Permission.MASTER,
    ):
        if isinstance(member, Member):
            member = member.id
        if Permission.get(member) >= override_level:
            return
        current = time.time()
        async with (await cls.get_lock()):
            last = cls.last_exec[member]
            if current - cls.last_exec[member][1] >= suspend_time:
                cls.last_exec[member] = (1, current)
                if member in cls.sent_alert:
                    cls.sent_alert.remove(member)
                return
            elif last[0] < max_exec:
                cls.last_exec[member] = (last[0] + 1, current)
                if member in cls.sent_alert:
                    cls.sent_alert.remove(member)
                return
            if member not in cls.sent_alert:
                cls.sent_alert.add(member)
            raise ExecutionStop()



import contextlib

from loguru import logger
from graia.ariadne.app import Ariadne
from sentry_sdk import capture_exception
from graia.ariadne.message.element import AtAll
from graia.broadcast.exceptions import ExecutionStop
from graia.ariadne.message.chain import MessageChain
from typing import Optional, TypeVar, Union, overload
from graia.ariadne.event.message import ActiveMessage
from graia.ariadne.model import Friend, Group, Member
from graia.ariadne.typing import SendMessageAction, SendMessageException
from graia.ariadne.exception import UnknownTarget, AccountMuted, RemoteException

from core.context import Context
from core import BOT_Status, Status
from core.bot_config import BotConfig

from .up_operation import delete_group

Exc_T = TypeVar("Exc_T", bound=SendMessageException)


class Safe(SendMessageAction):
    """
    安全发送的 SendMessage action
    """

    @overload
    @staticmethod
    async def exception(item) -> ActiveMessage:
        ...

    @overload
    async def exception(self, item) -> ActiveMessage:
        ...

    @staticmethod
    async def _handle(item: SendMessageException):

        push_type = "动态" if Context.push_type.get() == "dynamic" else "直播"
        push_id = Context.push_id.get()
        chain: MessageChain = item.send_data["message"]
        target: Union[Friend, Group, Member] = item.send_data["target"]
        ariadne = Ariadne.current()

        if isinstance(target, Group):
            if isinstance(item, UnknownTarget):
                logger.error(
                    f"[BiliBili推送] {push_type} {push_id} | 推送失败，找不到该群 {target.id}，正在取消订阅"
                )
                delete = await delete_group(target.id)
                logger.warning(f"[BiliBili推送] 已删除群 {target.id} 订阅的 {len(delete)} 个 UP")
                await ariadne.send_friend_message(
                    BotConfig.master,
                    MessageChain(
                        f"{push_type} {push_id} | 推送失败，找不到该群，已删除群 {target.id} 订阅的 {len(delete)} 个 UP"
                    ),
                )
                with contextlib.suppress(UnknownTarget):
                    await ariadne.quit_group(int(target.id))
            elif isinstance(item, AccountMuted):
                group = f"{target.name}（{target.id}）"
                msg = f"{push_type} {push_id} | 推送失败，Bot 在 {group} 被禁言"
                logger.warning(f"[BiliBili推送] {msg}")
                await ariadne.send_friend_message(BotConfig.master, MessageChain(msg))
            elif isinstance(item, RemoteException):
                if "resultType=46" in str(item):
                    msg = f"{push_type} {push_id} | 推送失败，Bot 被限制发送群聊消息（46 代码），请尽快处理后发送 /init 重新开启推送进程"
                    logger.error(f"[BiliBili推送] {msg}")
                    await ariadne.send_friend_message(BotConfig.master, MessageChain(msg))
                    BOT_Status.set_status(Status.INITIALIZED, True)
                    raise ExecutionStop()
                elif "resultType=110" in str(item):  # 110: 可能为群被封
                    logger.warning(f"[BiliBili推送] {push_type} {push_id} | 推送失败，Bot 因未知原因被移出群聊")
                    delete = await delete_group(target.id)
                    logger.warning(f"[BiliBili推送] 已删除群 {target.id} 订阅的 {len(delete)} 个 UP")
                    await ariadne.send_friend_message(
                        BotConfig.master,
                        MessageChain(
                            f"{push_type} {push_id} | 推送失败，Bot 因未知原因被移出群聊，已删除群 {target.id} 订阅的 {len(delete)} 个 UP"
                        ),
                    )
                    with contextlib.suppress(UnknownTarget):
                        await ariadne.quit_group(target.id)
                elif "reason=AT_ALL_LIMITED" in str(item):
                    msg = f"{push_type} {push_id} | 推送失败，Bot 在该群 @全体成员 次数已达上限"
                    logger.warning(f"[BiliBili推送] {msg}")
                    await ariadne.send_friend_message(
                        target.id,
                        MessageChain([e for e in chain.__root__ if not isinstance(e, AtAll)]),
                    )
                else:
                    capture_exception()
                    logger.exception(f"[BiliBili推送] {push_type} {push_id} | 推送失败，未知错误")

    @overload
    @staticmethod
    async def exception(s, i):
        ...

    @overload
    async def exception(s, i):  # sourcery skip: instance-method-first-arg-name
        ...

    async def exception(s: Union["Safe", Exc_T], i: Optional[Exc_T] = None):  # type: ignore
        # sourcery skip: instance-method-first-arg-name
        if not isinstance(s, Safe):
            return await Safe._handle(s)
        if i:
            return await Safe._handle(i)



import re

from loguru import logger

from .openai import openai_req, get_small_size_transcripts, get_summarise_prompt


async def subtitle_summarise(sub: list[str], title: str):
    """请求字幕总结"""
    small_size_transcripts = get_small_size_transcripts(sub)
    prompt = get_summarise_prompt(title, small_size_transcripts)
    logger.debug(prompt)
    return await openai_req(prompt)


async def column_summarise(cv_title: str, cv_text: str):
    """请求专栏总结"""
    sentences = re.split(r"[，。；,.;\n]+", cv_text)
    small_size_transcripts = get_small_size_transcripts(sentences)
    prompt = get_summarise_prompt(cv_title, small_size_transcripts)
    logger.debug(prompt)
    return await openai_req(prompt)



from __future__ import annotations

import time
from os import PathLike
from pathlib import Path
from typing import Literal, Optional

import httpx
from loguru import logger

from ..model.bcut_asr import (
    ResourceCompleteRspSchema,
    ResourceCreateRspSchema,
    ResultRspSchema,
    TaskCreateRspSchema,
)

__version__ = "0.0.2"

API_REQ_UPLOAD = "https://member.bilibili.com/x/bcut/rubick-interface/resource/create"  # 申请上传
API_COMMIT_UPLOAD = (
    "https://member.bilibili.com/x/bcut/rubick-interface/resource/create/complete"  # 提交上传
)
API_CREATE_TASK = "https://member.bilibili.com/x/bcut/rubick-interface/task"  # 创建任务
API_QUERY_RESULT = "https://member.bilibili.com/x/bcut/rubick-interface/task/result"  # 查询结果

SUPPORT_SOUND_FORMAT = Literal["flac", "aac", "m4a", "mp3", "wav"]


class APIError(Exception):
    "接口调用错误"

    def __init__(self, code, msg) -> None:
        self.code = code
        self.msg = msg
        super().__init__()

    def __str__(self) -> str:
        return f"{self.code}:{self.msg}"


class BcutASR:
    "必剪 语音识别接口"
    session: httpx.AsyncClient
    sound_name: str
    sound_bin: bytes
    sound_fmt: SUPPORT_SOUND_FORMAT
    __in_boss_key: str
    __resource_id: str
    __upload_id: str
    __upload_urls: list[str]
    __per_size: int
    __clips: int
    __etags: list[str]
    __download_url: str
    task_id: str

    def __init__(self, file: Optional[str | PathLike] = None) -> None:
        self.session = httpx.AsyncClient()
        self.task_id = ""
        self.__etags = []
        if file:
            self.set_data(file)

    def set_data(
        self,
        file: Optional[str | PathLike] = None,
        raw_data: Optional[bytes] = None,
        data_fmt: Optional[SUPPORT_SOUND_FORMAT] = None,
    ) -> None:
        "设置欲识别的数据"
        if file:
            if not isinstance(file, (str, PathLike)):
                raise TypeError("unknow file ptr")
            # 文件类
            file = Path(file)
            self.sound_bin = open(file, "rb").read()
            suffix = data_fmt or file.suffix[1:]
            self.sound_name = file.name
        elif raw_data:
            # bytes类
            self.sound_bin = raw_data
            suffix = data_fmt
            self.sound_name = f"{int(time.time())}.{suffix}"
        else:
            raise ValueError("none set data")
        if suffix not in SUPPORT_SOUND_FORMAT.__args__:
            raise TypeError("format is not support")
        self.sound_fmt = suffix
        logger.info(f"加载文件成功: {self.sound_name}")

    async def upload(self) -> None:
        "申请上传"
        if not self.sound_bin or not self.sound_fmt:
            raise ValueError("none set data")
        resp = await self.session.post(
            API_REQ_UPLOAD,
            data={
                "type": 2,
                "name": self.sound_name,
                "size": len(self.sound_bin),
                "resource_file_type": self.sound_fmt,
                "model_id": 7,
            },
        )
        resp.raise_for_status()
        resp = resp.json()
        if code := resp["code"]:
            raise APIError(code, resp["message"])
        resp_data = ResourceCreateRspSchema.parse_obj(resp["data"])
        self.__in_boss_key = resp_data.in_boss_key
        self.__resource_id = resp_data.resource_id
        self.__upload_id = resp_data.upload_id
        self.__upload_urls = resp_data.upload_urls
        self.__per_size = resp_data.per_size
        self.__clips = len(resp_data.upload_urls)
        logger.info(
            f"申请上传成功, 总计大小{resp_data.size // 1024}KB, "
            f"{self.__clips}分片, 分片大小{resp_data.per_size // 1024}KB: {self.__in_boss_key}"
        )
        await self.__upload_part()
        await self.__commit_upload()

    async def __upload_part(self) -> None:
        "上传音频数据"
        for clip in range(self.__clips):
            start_range = clip * self.__per_size
            end_range = (clip + 1) * self.__per_size
            logger.info(f"开始上传分片{clip}: {start_range}-{end_range}")
            resp = await self.session.put(
                self.__upload_urls[clip],
                data=self.sound_bin[start_range:end_range],
            )
            resp.raise_for_status()
            etag = resp.headers.get("Etag")
            self.__etags.append(etag)
            logger.info(f"分片{clip}上传成功: {etag}")

    async def __commit_upload(self) -> None:
        "提交上传数据"
        resp = await self.session.post(
            API_COMMIT_UPLOAD,
            data={
                "in_boss_key": self.__in_boss_key,
                "resource_id": self.__resource_id,
                "etags": ",".join(self.__etags),
                "upload_id": self.__upload_id,
                "model_id": 7,
            },
        )
        resp.raise_for_status()
        resp = resp.json()
        if code := resp["code"]:
            raise APIError(code, resp["message"])
        resp_data = ResourceCompleteRspSchema.parse_obj(resp["data"])
        self.__download_url = resp_data.download_url
        logger.info("提交成功")

    async def create_task(self) -> str:
        "开始创建转换任务"
        resp = await self.session.post(
            API_CREATE_TASK, json={"resource": self.__download_url, "model_id": "7"}
        )
        resp.raise_for_status()
        resp = resp.json()
        if code := resp["code"]:
            raise APIError(code, resp["message"])
        resp_data = TaskCreateRspSchema.parse_obj(resp["data"])
        self.task_id = resp_data.task_id
        logger.info(f"任务已创建: {self.task_id}")
        return self.task_id

    async def result(self, task_id: Optional[str] = None) -> ResultRspSchema:
        "查询转换结果"
        resp = await self.session.get(
            API_QUERY_RESULT, params={"model_id": 7, "task_id": task_id or self.task_id}
        )
        resp.raise_for_status()
        resp = resp.json()
        if code := resp["code"]:
            raise APIError(code, resp["message"])
        return ResultRspSchema.parse_obj(resp["data"])



from loguru import logger
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.scheduler.timers import crontabify
from graia.ariadne.message.chain import MessageChain
from graia.scheduler.saya.schema import SchedulerSchema

from ...core.bot_config import BotConfig
from ...utils.update_version import update_version


channel = Channel.current()


@channel.use(SchedulerSchema(crontabify("0 12 * * *")))
async def main(app: Ariadne):
    logger.info("[版本更新] 正在检查版本更新...")

    update = await update_version()

    if update:
        old_version, new_version = update
        logger.warning(f"[版本更新] 检测到新版本：{new_version} > {old_version}")
        await app.send_friend_message(
            BotConfig.master,
            MessageChain(f"检测到 BBot 版本更新：{new_version} > {old_version}，建议及时更新以避免出现问题并获取最新功能。"),
        )



import time

from loguru import logger
from typing import Optional, Union
from datetime import datetime, timedelta
from peewee import (
    Cast,
    Model,
    CharField,
    IntegerField,
    BooleanField,
    DateTimeField,
    SqliteDatabase,
)


data_version = 5
db = SqliteDatabase("data/data.db")


class BaseModel(Model):
    class Meta:
        database = db


class DynamicPush(BaseModel):
    """动态推送记录"""

    uid = CharField()
    uname = CharField()
    dyn_id = CharField(unique=True)
    dyn_type = CharField()
    dyn_text = CharField(null=True)
    push_groups = IntegerField()
    push_time = DateTimeField(default=datetime.now)

    class Meta:
        table_name = "dynamic_push"


class GroupPush(BaseModel):
    """群推送记录"""

    group = CharField()
    dyn_id = CharField()
    push_time = DateTimeField(default=datetime.now)

    class Meta:
        table_name = "group_push"


class LivePush(BaseModel):
    """直播推送记录"""

    uid = CharField()
    room_name = CharField(null=True)
    room_area_parent = CharField(null=True)
    room_area = CharField(null=True)
    status = BooleanField()
    push_groups = IntegerField()
    push_time = DateTimeField(default=datetime.now)

    class Meta:
        table_name = "live_push"


class SubList(BaseModel):
    """订阅列表"""

    uid = CharField()
    uname = CharField()
    nick = CharField(null=True)
    group = CharField()
    atall = BooleanField(default=False)
    live = BooleanField(default=True)
    live_tips = CharField(null=True)
    dynamic = BooleanField(default=True)
    cover_img = CharField(null=True)

    class Meta:
        table_name = "sub_list"


class ContentResolveArchive(BaseModel):
    """内容解析存档"""

    avid = CharField(null=True)
    cvid = CharField(null=True)
    title = CharField()
    content = CharField()
    jieba = CharField(null=True)
    openai = CharField(null=True)

    class Meta:
        table_name = "content_resolve_archive"


class TalkCount(BaseModel):
    """消息统计"""

    time = DateTimeField()
    count = IntegerField()

    class Meta:
        table_name = "talk_count"


class DataVersion(BaseModel):
    """数据库版本记录"""

    version = IntegerField()

    class Meta:
        table_name = "data_version"


db.create_tables(
    [DynamicPush, GroupPush, LivePush, SubList, TalkCount, DataVersion, ContentResolveArchive],
    safe=True,
)


if not DataVersion.select().exists():
    logger.info(f"数据库版本记录不存在，正在创建，当前版本：{data_version}")
    DataVersion(version=data_version).save()
elif DataVersion.get().version != data_version:
    logger.info(f"数据库版本不匹配，当前当前最新版本：{data_version}，正在更新")
    while DataVersion.get().version != data_version:
        if DataVersion.get().version == 1:
            logger.info("当前数据版本为 1，正在更新至 2")
            # 将 LivePush 表中的 statu 字段改为 status
            db.execute_sql("ALTER TABLE live_push RENAME COLUMN statu TO status")
            DataVersion.update(version=2).execute()
        elif DataVersion.get().version == 2:
            logger.info("当前数据版本为 2，正在更新至 3")
            # 在 SubList 表中添加 live_tips 字段，允许为空
            db.execute_sql("ALTER TABLE sub_list ADD COLUMN live_tips VARCHAR(255) NULL")
            DataVersion.update(version=3).execute()
        elif DataVersion.get().version == 3:
            logger.info("当前数据版本为 3，正在更新至 4")
            # 在 SubList 表中添加 cover_img 字段，允许为空
            db.execute_sql("ALTER TABLE sub_list ADD COLUMN cover_img VARCHAR(255) NULL")
            DataVersion.update(version=4).execute()
        elif DataVersion.get().version == 4:
            logger.info("当前数据版本为 4，正在更新至 5")
            DataVersion.update(version=5).execute()

    logger.success("数据库更新完成")
    time.sleep(2)


def insert_dynamic_push(
    uid: Union[str, int],
    uname: str,
    dyn_id: Union[str, int],
    dyn_type,
    dyn_text: str,
    push_groups: int,
):
    """在动态推送表中插入一条记录"""
    DynamicPush(
        uid=str(uid),
        uname=uname,
        dyn_id=str(dyn_id),
        dyn_type=dyn_type,
        dyn_text=dyn_text,
        push_groups=push_groups,
    ).save()


def insert_live_push(
    uid: Union[str, int],
    status: bool,
    push_groups: int,
    room_name: str = "",
    room_area_parent: str = "",
    room_area: str = "",
):
    """在直播推送表中插入一条记录"""
    LivePush(
        uid=uid,
        room_name=room_name,
        room_area_parent=room_area_parent,
        room_area=room_area,
        status=status,
        push_groups=push_groups,
    ).save()


def is_dyn_pushed(pushid: Union[str, int]) -> bool:
    """查询某个动态是否推送过"""
    return bool(DynamicPush.select().where(DynamicPush.dyn_id == str(pushid)).exists())


def add_sub(uid: Union[str, int], uname: str, group: Union[str, int]):
    """添加订阅"""
    SubList(uid=str(uid), uname=uname, group=group).save()


def get_all_uid() -> list[str]:
    """获取所有uid"""
    return [i.uid for i in SubList.select(SubList.uid).distinct()]


def get_sub_by_group(group: Union[str, int]) -> list[SubList]:
    """根据群组获取订阅列表"""
    return list(
        SubList.select().where(SubList.group == group).order_by(Cast(SubList.uid, "int"))
    )


def get_sub_by_uid(uid: Union[str, int]) -> list[SubList]:
    """根据uid获取订阅该uid的群"""
    return list(SubList.select().where(SubList.uid == str(uid)).order_by(SubList.group))


def get_sub_data(uid: Union[str, int], group: Union[str, int]) -> SubList:
    """获取订阅数据"""
    return SubList.get(SubList.uid == str(uid), SubList.group == str(group))


def set_uid_name(uid: Union[str, int], uname: str):
    """设置用户名"""
    SubList.update(uname=uname).where(SubList.uid == str(uid)).execute()


def uid_exists(uid: Union[str, int]) -> bool:
    """检查uid是否存在数据库中"""
    return bool(SubList.select().where(SubList.uid == str(uid)).exists())


def uid_in_group(uid: Union[str, int], group: Union[str, int]) -> bool:
    """检查uid是否在该群订阅中"""
    return bool(
        SubList.select().where(SubList.uid == str(uid), SubList.group == str(group)).exists()
    )


def unsub_uid_by_group(uid: Union[str, int], group: Union[str, int]):
    """取消该uid在该群的订阅"""
    SubList.delete().where(SubList.uid == str(uid), SubList.group == str(group)).execute()


def delete_sub_by_uid(uid: Union[str, int]):
    """删除该uid的所有订阅"""
    SubList.delete().where(SubList.uid == str(uid)).execute()


def add_talk_count():
    """添加聊天计数"""
    now_time = datetime.now().replace(minute=0, second=0, microsecond=0)
    # 如果当前时间已经存在记录，则计数加一，否则新建一条记录
    if TalkCount.select().where(TalkCount.time == now_time).exists():
        TalkCount.update(count=TalkCount.count + 1).where(TalkCount.time == now_time).execute()
    else:
        TalkCount(time=now_time, count=1).save()


def get_talk_count(from_time: datetime, to_time: datetime) -> list[dict[str, int]]:
    """获取指定范围内的每个整点小时的消息量"""
    return [
        {
            "time": int(x.timestamp()),
            "count": TalkCount.get(TalkCount.time == x).count
            if TalkCount.select().where(TalkCount.time == x).exists()
            else 0,
        }
        for x in [
            from_time.replace(minute=0, second=0, microsecond=0) + timedelta(hours=i)
            for i in range(int((to_time - from_time).total_seconds() // 3600))
        ]
    ]


def get_push_count(
    from_time: Optional[datetime] = None, to_time: Optional[datetime] = None
) -> int:
    """获取并返回指定范围内的推送计数"""
    if from_time and to_time:
        return (
            DynamicPush.select()
            .where(DynamicPush.push_time >= from_time, DynamicPush.push_time <= to_time)
            .count()
        )
    else:
        return DynamicPush.select().count()


# 添加动态推送至群的记录
def insert_dyn_push_to_group(dyn_id: Union[str, int], group: Union[str, int]):
    GroupPush(dyn_id=str(dyn_id), group=str(group)).save()


# 判断该动态是否在该群推送过
def is_dyn_pushed_in_group(dyn_id: Union[str, int], group: Union[str, int]) -> bool:
    return bool(
        GroupPush.select()
        .where(GroupPush.dyn_id == str(dyn_id), GroupPush.group == str(group))
        .exists()
    )


# 根据 uid 修改封面
def update_cover_by_id(uid: Union[str, int], cover: Optional[str]):
    SubList.update(cover_img=cover).where(SubList.uid == str(uid)).execute()
    return cover or SubList.get(SubList.uid == str(uid)).cover_img


class ContentResolveData:
    """内容解析存档数据操作"""

    def __init__(
        self, aid: Optional[Union[str, int]] = None, cid: Optional[Union[str, int]] = None
    ):
        self.aid = aid
        self.cid = cid
        self.sid = str(aid or cid)
        self.title: Optional[str] = None
        self.content: Optional[str] = None
        self.jieba: Optional[str] = None
        self.openai: Optional[str] = None

        if self.check_exists():
            data = self.get_data()
            self.title = data.title
            self.content = data.content
            self.jieba = data.jieba
            self.openai = data.openai

    def check_exists(self) -> bool:
        """检查是否存在"""
        return bool(
            ContentResolveArchive.select()
            .where(
                ContentResolveArchive.avid == self.sid or ContentResolveArchive.cvid == self.sid
            )
            .exists()
        )

    def get_data(self) -> Optional[ContentResolveArchive]:
        """获取数据"""
        if self.check_exists():
            return ContentResolveArchive.get(
                ContentResolveArchive.avid == self.sid or ContentResolveArchive.cvid == self.sid
            )

    def save(self):
        """保存数据"""
        if self.check_exists():
            ContentResolveArchive.update(
                title=self.title,
                content=self.content,
                jieba=self.jieba,
                openai=self.openai,
            ).where(
                ContentResolveArchive.avid == self.sid or ContentResolveArchive.cvid == self.sid
            ).execute()
        else:
            ContentResolveArchive(
                avid=self.sid if self.aid else None,
                cvid=self.sid if self.cid else None,
                title=self.title,
                content=self.content,
                jieba=self.jieba,
                openai=self.openai,
            ).save()



import re
import string
import random


def num_fmt(num: int):
    if num < 10000:
        return str(num)
    elif num < 100000000:
        return ("%.2f" % (num / 10000)) + "万"
    else:
        return ("%.2f" % (num / 100000000)) + "亿"


def get_cut_str(str_in, cut):
    """
    自动断行，用于 Pillow 等不会自动换行的场景
    """
    punc = """，,、。.？?）》】“"‘'；;：:！!·`~%^& """
    si = 0
    i = 0
    next_str = str_in
    str_list = []

    while re.search(r"\n\n\n\n\n", next_str):
        next_str = re.sub(r"\n\n\n\n\n", "\n", next_str)
    for s in next_str:
        si += 1 if s in string.printable else 2
        i += 1
        if next_str == "":
            break
        elif next_str[0] == "\n":
            next_str = next_str[1:]
        elif s == "\n":
            str_list.append(next_str[: i - 1])
            next_str = next_str[i - 1:]
            si = 0
            i = 0
            continue
        if si > cut:
            try:
                if next_str[i] in punc:
                    i += 1
            except IndexError:
                str_list.append(next_str)
                return str_list
            str_list.append(next_str[:i])
            next_str = next_str[i:]
            si = 0
            i = 0
    str_list.append(next_str)
    i = 0
    non_wrap_str = []
    for p in str_list:
        if p == "":
            break
        elif p[-1] == "\n":
            p = p[:-1]
        non_wrap_str.append(p)
        i += 1
    return non_wrap_str


def getCutStr(str_in, cut):
    cut_str = str_in
    si = 0
    i = 0
    for s in str_in:
        si += 2 if "\u4e00" <= s <= "\u9fff" else 1
        i += 1
        if si > cut:
            cut_str = f"{str_in[:i]}...."
            break

    return cut_str


def generate_verify_code():
    return str(random.randint(100000, 999999))



import json
from pathlib import Path

subgroup_config_file = Path("data/subgroup_list.json")

subgroups = []
group_names = []

if subgroup_config_file.exists():
    data = json.loads(subgroup_config_file.read_text())
    for group_name, subgroup in data["sublist"].items():
        group_names.append(group_name)
        subgroups.append(subgroup)
else:
    subgroup_config_file.write_text(json.dumps({"sublist": {}}, ensure_ascii=False))


def get_subgroup_list() -> tuple[list, list]:
    """
    用于返回订阅组名称列表与订阅列表
    """
    return group_names, subgroups


def write_to_json():
    """
    将订阅组列表与订阅列表一一对应组装放入json文件
    """
    subgroup_config_file.write_text(
        json.dumps({"sublist": dict(zip(group_names, subgroups))}, ensure_ascii=False)
    )


class SubGroup:
    group_name: str

    def __init__(self, in_group_name: str):
        """
        init
        """
        self.group_name = in_group_name

    def is_in_group_names(self) -> bool:
        """
        判断该订阅组名称是否在订阅组列表中
        """
        return self.group_name in group_names

    def add_to_group_names(self) -> bool:
        """
        将订阅组名称添加到订阅组列表中
        """
        if self.is_in_group_names():
            return False

        group_names.append(self.group_name)
        subgroups.append([])

        write_to_json()

        return True

    def remove_from_group_names(self) -> bool:
        """
        从订阅组列表中移除该订阅组名称
        """
        if not self.is_in_group_names():
            return False
        idx = group_names.index(self.group_name)  # 获取对应的索引
        group_names.remove(self.group_name)

        subgroups.pop(idx)  # 根据索引移除列表

        write_to_json()

        return True

    def add_to_subGroups(self, up_list: list) -> bool:
        """
        将获取到的up主列表添加到对应的订阅组中
        """
        idx = group_names.index(self.group_name)  # 获取对应的索引
        uplist: list = subgroups[idx]  # 获取对应索引的订阅列表

        for up in uplist:  # 判断是否已经有相同的uid
            if up in up_list:
                up_list.remove(up)
        uplist += up_list
        if len(uplist) > 12:
            return False

        subgroups[idx] = uplist  # 替换订阅列表

        write_to_json()

        return True

    def remove_from_subGroup_ups(self, up_id: int) -> bool:
        """
        从对应订阅组中移除指定的up主
        """
        idx = group_names.index(self.group_name)  # 获取对应的索引
        uplist: list = subgroups[idx]  # 获取对应索引的订阅列表

        if not uplist:  # 如果为空则不需要移除
            return False

        if up_id in uplist:
            uplist.remove(up_id)

        write_to_json()

        return True



import asyncio
import re
from io import BytesIO
from pathlib import Path

from graia.ariadne.app import Ariadne
from PIL import Image, ImageDraw, ImageFont

from .fonts_provider import get_font_sync
from .strings import get_cut_str

data_path = Path("data", "render")
data_path.mkdir(parents=True, exist_ok=True)
font = ImageFont.truetype(str(get_font_sync("sarasa-mono-sc-semibold.ttf")), size=20)


async def text2image(text: str, cut=64) -> bytes:
    return await asyncio.to_thread(_create_image, text, cut)


def _create_image(text: str, cut: int) -> bytes:
    cut_str = "\n".join(get_cut_str(text, cut))
    textx, texty = font.getsize_multiline(cut_str)
    image = Image.new("RGB", (textx + 40, texty + 40), (235, 235, 235))
    draw = ImageDraw.Draw(image)
    draw.text((20, 20), cut_str, font=font, fill=(31, 31, 33))
    imageio = BytesIO()
    image.save(
        imageio,
        format="JPEG",
        quality=90,
        subsampling=2,
        qtables="web_high",
    )
    return imageio.getvalue()


async def rich_text2image(data: str):
    from io import BytesIO

    from dynamicadaptor.Content import RichTextDetail, Text
    from minidynamicrender.DynConfig import ConfigInit
    from minidynamicrender.DynText import DynTextRender

    from .fonts_provider import get_font

    config = ConfigInit(
        data_path=str(data_path),
        font_path={
            "text": str(await get_font("HarmonyOS_Sans_SC_Medium.ttf")),
            "extra_text": str(await get_font("sarasa-mono-sc-bold.ttf")),
            "emoji": str(await get_font("nte.ttf")),
        },
    )
    if config.dyn_color and config.dyn_font and config.dy_size:
        render = DynTextRender(
            config.static_path, config.dyn_color, config.dyn_font, config.dy_size
        )
        image = await render.run(
            Text(
                text=data,
                topic=None,
                rich_text_nodes=[
                    RichTextDetail(
                        type="RICH_TEXT_NODE_TYPE_TEXT", text=data, orig_text=data, emoji=None
                    )
                ],
            )
        )
        if image:
            bio = BytesIO()
            image.convert("RGB").save(bio, "jpeg", optimize=True)
            return bio.getvalue()


async def browser_text2image(data: str):
    import jinja2
    from graiax.playwright.interface import PlaywrightContext

    from .browser_shot import fill_font

    app = Ariadne.current()
    browser_context = app.launch_manager.get_interface(PlaywrightContext).context

    src = "openai"
    data = r"\n".join(data.splitlines())

    summary = Path(__file__).parent.parent.joinpath("static", "summary")
    template_env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(summary),
        enable_async=True,
    )
    template_path = f"file:///{summary.joinpath('index.html').absolute()}".replace(
        "////", "///"
    )
    template = template_env.get_template("index.html")
    html = await template.render_async(
        **{
            "data": data,
            "src": src,
        }
    )

    async with await browser_context.new_page() as page:
        await page.route(re.compile("^https://fonts.bbot/(.+)$"), fill_font)
        await page.set_viewport_size({"width": 800, "height": 2000})
        await page.goto(template_path)
        await page.set_content(html, wait_until="networkidle")
        await page.wait_for_timeout(5)
        img_raw = await page.get_by_alt_text("main").screenshot(
            type="png",
        )
    return img_raw



import datetime


def calc_time_total(t):
    t = int(t * 1000)
    if t < 5000:
        return f"{t} 毫秒"

    timedelta = datetime.timedelta(seconds=t // 1000)
    day = timedelta.days
    hour, mint, sec = tuple(int(n) for n in str(timedelta).split(",")[-1].split(":"))

    total = ""
    if day:
        total += f"{day} 天 "
    if hour:
        total += f"{hour} 小时 "
    if mint:
        total += f"{mint} 分钟 "
    if sec and not day and not hour:
        total += f"{sec} 秒 "
    return total



from enum import Enum
from pydantic import BaseModel


class ASRDataSeg(BaseModel):
    "文字识别-断句"

    class ASRDataWords(BaseModel):
        "文字识别-逐字"
        label: str
        start_time: int
        end_time: int
        confidence: int

    start_time: int
    end_time: int
    transcript: str
    words: list[ASRDataWords]
    confidence: int

    def to_srt_ts(self) -> str:
        "转换为srt时间戳"

        def _conv(ms: int) -> tuple[int, int, int, int]:
            return ms // 3600000, ms // 60000 % 60, ms // 1000 % 60, ms % 1000

        s_h, s_m, s_s, s_ms = _conv(self.start_time)
        e_h, e_m, e_s, e_ms = _conv(self.end_time)
        return f"{s_h:02d}:{s_m:02d}:{s_s:02d},{s_ms:03d} --> {e_h:02d}:{e_m:02d}:{e_s:02d},{e_ms:03d}"

    def to_lrc_ts(self) -> str:
        "转换为lrc时间戳"

        def _conv(ms: int) -> tuple[int, int, int]:
            return ms // 60000, ms // 1000 % 60, ms % 1000 // 10

        s_m, s_s, s_ms = _conv(self.start_time)
        return f"[{s_m:02d}:{s_s:02d}.{s_ms:02d}]"


class ASRData(BaseModel):
    "语音识别结果"
    utterances: list[ASRDataSeg]
    version: str

    def __iter__(self):
        "iter穿透"
        return iter(self.utterances)

    def has_data(self) -> bool:
        "是否识别到数据"
        return len(self.utterances) > 0

    def to_txt(self) -> str:
        "转成txt格式字幕 (无时间标记)"
        return "\n".join(seg.transcript for seg in self.utterances)

    def to_srt(self) -> str:
        "转成srt格式字幕"
        return "\n".join(
            f"{n}\n{seg.to_srt_ts()}\n{seg.transcript}\n"
            for n, seg in enumerate(self.utterances, 1)
        )

    def to_lrc(self) -> str:
        "转成lrc格式字幕"
        return "\n".join(f"{seg.to_lrc_ts()}{seg.transcript}" for seg in self.utterances)

    def to_ass(self) -> str:
        ...


class ResourceCreateRspSchema(BaseModel):
    "上传申请响应"
    resource_id: str
    title: str
    type: int
    in_boss_key: str
    size: int
    upload_urls: list[str]
    upload_id: str
    per_size: int


class ResourceCompleteRspSchema(BaseModel):
    "上传提交响应"
    resource_id: str
    download_url: str


class TaskCreateRspSchema(BaseModel):
    "任务创建响应"
    resource: str
    result: str
    task_id: str  # 任务id


class ResultStateEnum(Enum):
    "任务状态枚举"
    STOP = 0  # 未开始
    RUNING = 1  # 运行中
    ERROR = 3  # 错误
    COMPLETE = 4  # 完成


class ResultRspSchema(BaseModel):
    "任务结果查询响应"
    task_id: str  # 任务id
    result: str  # 结果数据-json
    remark: str  # 任务状态详情
    state: ResultStateEnum  # 任务状态

    def parse(self) -> ASRData:
        "解析结果数据"
        return ASRData.parse_raw(self.result)



import re

from graia.ariadne.message.chain import MessageChain
from graia.ariadne.message.element import Image, Voice, FlashImage

from .b23_extract import b23_extract


async def extract_bilibili_info(message: MessageChain):
    if message.only(Image) or message.has(Voice) or message.has(FlashImage):
        return

    message_str = message.as_persistent_string(binary=False, exclude=[Image])
    p = re.compile(r"av(\d{1,16})|BV(1[A-Za-z0-9]{2}4.1.7[A-Za-z0-9]{2})|cv(\d{1,16})")
    if not p.search(message_str) and ("b23.tv" in message_str or "b23.wtf" in message_str):
        message_str = await b23_extract(message_str) or message_str
    if bili_number := p.search(message_str):
        bili_number = bili_number[0]
        return bili_number
    else:
        return



from typing import Optional
from starlette import status
from datetime import datetime
from pydantic import BaseModel
from graia.ariadne.model import MemberPerm

from ..model.config import _BotConfig


class Token(BaseModel):
    access_token: str
    expires_at: Optional[datetime] = None
    token_type: str = "bearer"


class Info(BaseModel):
    qq: str
    permission: int
    token: Token


class BaseResponse(BaseModel):
    code: int = status.HTTP_200_OK
    message: str = "Success"


class AuthResponse(BaseResponse):
    data: Optional[Info]


class KeyResponse(BaseResponse):
    data: str


class InfoResponse(BaseResponse):
    data: Info


class GroupItem(BaseModel):
    id: int
    name: str
    member_count: int
    follow_count: int
    is_vip: bool


class GroupListResponse(BaseResponse):
    data: Optional[list[GroupItem]]


class MemberItem(BaseModel):
    id: int
    name: str
    permission: MemberPerm
    special_title: Optional[str]
    join_timestamp: Optional[int]
    last_speak_timestamp: Optional[int]
    mute: Optional[int]


class MemberListResponse(BaseResponse):
    data: Optional[list[MemberItem]]


class FollowItem(BaseModel):
    uid: str
    uname: str
    nick: Optional[str]
    atall: bool
    live: bool
    dynamic: bool


class FollowListResponse(BaseResponse):
    data: Optional[list[FollowItem]]


class AddFollowResponse(BaseResponse):
    pass


class TalkCount(BaseModel):
    time: int
    count: int


class HomeItem(BaseModel):
    group_count: int
    friend_count: int
    uid_count: int
    push_count: int
    all_push_count: int
    talk_count: list[TalkCount]


class HomeResponse(BaseResponse):
    data: HomeItem


class ConfigResponse(BaseResponse):
    data: _BotConfig



import asyncio

from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import GroupMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    FullMatch,
    RegexResult,
    WildcardMatch,
)

from ...core.control import Permission

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[Twilight([FullMatch("/公告"), "anything" @ WildcardMatch()])],
        decorators=[Permission.require(Permission.MASTER)],
    )
)
async def main(app: Ariadne, anything: RegexResult):

    if anything.matched:
        msg = anything.result.display
        sent = []
        for group in await app.get_group_list():
            if group.id in sent:
                continue
            await app.send_group_message(group, MessageChain(f"公告 - {group.id}：\n{msg}"))
            sent.append(group.id)
            await asyncio.sleep(2)



from lxml import etree
from lxml.etree import _Element, _ElementUnicodeResult

from ..model.exception import AbortError

from .bilibili_request import hc

XPATH = "//p//text() | //h1/text() | //h2/text() | //h3/text() | //h4/text() | //h5/text() | //h6/text()"


async def get_cv(cvid: str) -> tuple[str, str]:
    cv = await hc.get(f"https://www.bilibili.com/read/cv{cvid}", follow_redirects=True)
    if cv.status_code != 200:
        raise AbortError("专栏获取失败")
    elif cv.url != f"https://www.bilibili.com/read/cv{cvid}":
        if cv.url == "https://www.bilibili.com/read/error":
            raise AbortError("专栏不存在")
        else:
            raise AbortError("专栏获取失败")
    cv.encoding = "utf-8"
    cv = cv.text

    http_parser: _Element = etree.fromstring(cv, etree.HTMLParser(encoding="utf-8"))
    title: str = http_parser.xpath('//h1[@class="title"]/text()')[0]
    main_article: _Element = http_parser.xpath('//div[@id="read-article-holder"]')[0]
    plist: _ElementUnicodeResult = main_article.xpath(XPATH)
    text_list = [text.strip() for text in plist if text.strip()]
    return title, "\n".join(text_list)



from graia.ariadne.app import Ariadne
from fastapi import APIRouter, Depends, status
from graia.ariadne.message.chain import MessageChain

from .auth import verify_token, can_manage_group
from ....core.data import get_sub_by_group, uid_in_group
from ....utils.up_operation import subscribe_uid, unsubscribe_uid
from ....model.fastapi import Info, FollowListResponse, FollowItem, AddFollowResponse


router = APIRouter(tags=["Follow"])


@router.get("/group_follow/{group}", summary="获取群关注列表")
async def get_group_follows(group: int, info: Info = Depends(verify_token)):
    return (
        FollowListResponse(
            data=[
                FollowItem(
                    uid=str(x.uid),
                    uname=str(x.uname),
                    nick=str(x.nick or ""),
                    atall=bool(x.atall),
                    live=bool(x.live),
                    dynamic=bool(x.dynamic),
                )
                for x in get_sub_by_group(group)
            ]
        )
        if await can_manage_group(info, group)
        else FollowListResponse(
            code=status.HTTP_403_FORBIDDEN, message="Permission denied", data=[]
        )
    )


@router.put("/group_follow/{group}", summary="关注 UP")
async def add_group_follow(
    group: int, uid: str, uname: str, info: Info = Depends(verify_token)
):
    if not await can_manage_group(info, group):
        return AddFollowResponse(code=status.HTTP_403_FORBIDDEN, message="Permission denied")
    if uid_in_group(uid, group):
        return AddFollowResponse(code=6400, message=f"本群已订阅 UP {uname}（{uid}），请勿重复订阅")
    sub = await subscribe_uid(uid, group)
    if not uid_in_group(uid, group):
        return AddFollowResponse(code=6400, message=sub)
    app = Ariadne.current()
    await app.send_group_message(group, MessageChain(f"管理员已通过 Web 关注 UP {uname}（{uid}）"))
    return AddFollowResponse(message=sub)


@router.delete("/group_follow/{group}", summary="取消关注 UP")
async def del_group_follow(
    group: int, uid: str, uname: str, info: Info = Depends(verify_token)
):
    if not await can_manage_group(info, group):
        return AddFollowResponse(code=status.HTTP_403_FORBIDDEN, message="Permission denied")
    if not uid_in_group(uid, group):
        return AddFollowResponse(code=6400, message=f"本群未订阅该 UP {uname}（{uid}）")
    unsub = await unsubscribe_uid(uid, group)
    if uid_in_group(uid, group):
        return AddFollowResponse(code=6400, message=unsub)
    app = Ariadne.current()
    await app.send_group_message(group, MessageChain(f"管理员已通过 Web 取消关注 UP {uname}（{uid}）"))
    return AddFollowResponse(message=unsub)



import sys
import asyncio

from loguru import logger
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from sentry_sdk import capture_exception
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.lifecycle import AccountLaunch
from graia.saya.builtins.broadcast.schema import ListenerSchema

from ...core.bot_config import BotConfig
from ...utils.bilibili_request import hc
from ...utils.detect_package import is_full
from ...utils.update_version import update_version

channel = Channel.current()


@channel.use(ListenerSchema(listening_events=[AccountLaunch]))
async def main(app: Ariadne):
    """
    Graia 成功启动并链接到 Mirai HTTP API 后执行的函数
    """
    if BotConfig.update_check:
        update = await update_version()
        if update:
            old_version, new_version = update
            logger.warning(f"[版本更新] 检测到新版本：{new_version} > {old_version}")
            await app.send_friend_message(
                BotConfig.master, MessageChain(f"检测到新版本：{new_version} > {old_version}")
            )
        else:
            logger.success("[版本更新] 当前版本为最新版本")

    if is_full and BotConfig.Bilibili.use_browser:
        from graiax.playwright.interface import PlaywrightContext

        try:
            logger.info("[Playwright] 正在获取浏览器版本")
            browser_context = app.launch_manager.get_interface(PlaywrightContext)
            if not BotConfig.Bilibili.mobile_style:
                await browser_context.context.add_cookies(
                    [
                        {
                            "name": "hit-dyn-v2",
                            "value": "1",
                            "domain": ".bilibili.com",
                            "path": "/",
                        }
                    ]
                )

            async with browser_context.page() as page:
                version = await page.evaluate("navigator.appVersion")
                logger.info(f"[BiliBili推送] 浏览器启动完成，当前版本 {version}")
                logger.debug(await browser_context.context.cookies())
        except Exception as e:
            capture_exception(e)
            logger.error(f"[BiliBili推送] 浏览器启动失败 {e}")
            sys.exit(1)

    logger.info("[BiliBili推送] 正在获取首页 Cookie")

    for _ in range(3):
        try:
            await hc.get("https://bilibili.com/", follow_redirects=True)
            break
        except Exception as e:
            logger.error(f"[BiliBili推送] 获取首页 Cookie 失败 {e}，正在重试")
            await asyncio.sleep(2)
    else:
        logger.error("[BiliBili推送] 获取首页 Cookie 失败，已达最大重试次数")
        sys.exit(1)

    logger.info(f"[BiliBili推送] 获取首页 Cookie 成功，{hc.cookies}")

    logger.info("Graia 成功启动")
    group_list = await app.get_group_list()
    group_num = len(group_list)
    master = await app.get_friend(BotConfig.master)
    if not master:
        logger.error(f"当前未添加主人好友（{BotConfig.master}），请手动添加")
        sys.exit(1)
    await app.send_friend_message(
        BotConfig.master,
        MessageChain(
            "BBot-Graia 成功启动。",
            f"\n当前 {BotConfig.name} 共加入了 {group_num} 个群",
        ),
    )

    if BotConfig.Debug.enable:
        debug_msg = []
        for group in BotConfig.Debug.groups:
            debug_group = await app.get_group(group)
            debug_msg.append(
                f"{debug_group.id}（{debug_group.name}）" if debug_group else f"{group}（当前未加入该群）"
            )
        await app.send_friend_message(
            BotConfig.master,
            MessageChain(
                "当前为 Debug 模式，将仅接受\n",
                "\n".join(debug_msg),
                f"\n以及 {master.nickname}（{master.id}） 的消息",
            ),
        )



import os

from pathlib import Path
from loguru import logger
from fastapi import status
from fastapi import FastAPI
from fastapi.responses import Response
from playwright.async_api import async_playwright
from playwright.async_api._generated import BrowserContext
from graiax.playwright.installer import install_playwright

from ..utils.browser_shot import screenshot
from ..utils.fonts_provider import font_init
from ..utils.detect_package import is_package


DYNAMIC_SHOT_CACHE = Path("data", "cache")
DYNAMIC_SHOT_CACHE.mkdir(parents=True, exist_ok=True)
os.environ["PLAYWRIGHT_BROWSERS_PATH"] = (
    "0" if is_package else Path(__file__).parent.parent.joinpath("static", "browser").as_posix()
)
PLAYWRIGIT: BrowserContext = None  # type: ignore
app = FastAPI(title="BBot Playwright API", version="0.1.0")


@app.on_event("startup")
async def init_playwright():
    global PLAYWRIGIT
    logger.info("正在下载字体...")
    font_init()
    logger.success("字体下载完成！")

    await install_playwright(browser_type="chromium")
    pw = await async_playwright().start()
    chrome = await pw.chromium.launch_persistent_context(
        Path("data").joinpath("browser"),
        device_scale_factor=1.5,
        user_agent=(
            "Mozilla/5.0 (Linux; Android 10; RMX1911) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/100.0.4896.127 Mobile Safari/537.36"
        ),
        # headless=False,
    )
    PLAYWRIGIT = chrome
    logger.info("[Playwright] 正在获取浏览器版本")
    if len(PLAYWRIGIT.pages) > 0:
        page = PLAYWRIGIT.pages[0]
    else:
        page = await PLAYWRIGIT.new_page()
    version = await page.evaluate("navigator.appVersion")
    logger.info(f"[BiliBili推送] 浏览器启动完成，当前版本 {version}")
    logger.debug(await PLAYWRIGIT.cookies())


@app.get("/dynamic/{dynid}")
async def get_up(dynid: str):
    imagefile = DYNAMIC_SHOT_CACHE.joinpath(f"{dynid}.jpg")
    if imagefile.exists():
        logger.info(f"动态 {dynid} 已缓存")
        page_shot = imagefile.read_bytes()
    else:
        page_shot = await screenshot(dynid, PLAYWRIGIT, False)

    if page_shot:
        imagefile.write_bytes(page_shot)
        return Response(page_shot)
    else:
        return Response(status_code=status.HTTP_400_BAD_REQUEST)



from pathlib import Path
from typing import Union
from fastapi import FastAPI
from fastapi.responses import FileResponse
from fastapi.staticfiles import StaticFiles
from fastapi.middleware.cors import CORSMiddleware
from launart import Launchable, ExportInterface, Launart
from graia.amnesia.transport.common.asgi import ASGIHandlerProvider

from .api.router import router

STATIC_PATH = Path(__file__).parent.joinpath("static")


async def root():
    return FileResponse(STATIC_PATH.joinpath("html", "index.html"))


async def status():
    return {"status": "ok"}


class BotWebService(Launchable):
    id: str = "webapi/bbot"

    @property
    def required(self) -> set[Union[str, type[ExportInterface]]]:
        return {ASGIHandlerProvider}

    @property
    def stages(self):
        return {"preparing"}

    async def launch(self, launart: Launart):
        async with self.stage("preparing"):
            app: FastAPI = launart.get_interface(ASGIHandlerProvider).get_asgi_handler()
            app.add_middleware(
                CORSMiddleware,
                allow_origins=["*"],
                allow_credentials=True,
                allow_methods=["*"],
                allow_headers=["*"],
            )
            app.include_router(router, prefix="/api")
            app.get("/api/status")(status)
            app.get("/")(root)
            app.mount(
                "/",
                StaticFiles(directory=STATIC_PATH.joinpath("html")),
                name="static",
            )



import asyncio
import contextlib

from typing import Union
from loguru import logger
from bilireq.grpc.dynamic import grpc_get_user_dynamics

from ..core import BOT_Status, Status
from ..core.bot_config import BotConfig
from ..core.group_config import GroupPermission
from ..core.data import (
    add_sub,
    uid_exists,
    get_sub_data,
    set_uid_name,
    uid_in_group,
    get_sub_by_uid,
    get_sub_by_group,
    delete_sub_by_uid,
    unsub_uid_by_group,
)

from .bilibili_request import relation_modify


async def subscribe_uid(uid: Union[str, int], groupid: Union[str, int]):
    """在某个群订阅某个 up"""
    uid = str(uid)
    groupid = str(groupid)
    gp = GroupPermission(int(groupid))
    BOT_Status.set_status(Status.SUBSCRIBE_IDLE, False)
    while not BOT_Status.check_status(Status.DYNAMIC_IDLE) and BotConfig.Bilibili.use_login:
        await asyncio.sleep(0.1)

    if not int(uid):
        BOT_Status.set_status(Status.SUBSCRIBE_IDLE, True)
        return "Bot 状态异常，订阅失败，请稍后再试"
    r = await grpc_get_user_dynamics(int(uid))
    if not r:
        BOT_Status.set_status(Status.SUBSCRIBE_IDLE, True)
        return f"该 UP（{uid}）状态异常，订阅失败"
    try:
        if len(r.list) == 1:
            dyn = r.list[0]
        elif r.list[0].modules[0].module_author.is_top:
            dyn = r.list[1]
        else:
            dyn = r.list[0]
        up_name = dyn.modules[0].module_author.author.name
    except IndexError:
        BOT_Status.set_status(Status.SUBSCRIBE_IDLE, True)
        return f"该 UP（{uid}）未发送任何动态，订阅失败"
    if uid_in_group(uid, groupid):
        BOT_Status.set_status(Status.SUBSCRIBE_IDLE, True)
        return f"本群已订阅 UP {up_name}（{uid}），请勿重复订阅"
    if len(get_sub_by_group(groupid)) >= BotConfig.max_subsubscribe and not gp.is_vip():
        BOT_Status.set_status(Status.SUBSCRIBE_IDLE, True)
        return f"每个群聊最多仅可订阅 {BotConfig.max_subsubscribe} 个 UP"
    need_sub = not uid_exists(uid)
    add_sub(uid, up_name, groupid)

    if need_sub and BotConfig.Bilibili.use_login:
        resp = await relation_modify(uid, 1)
        if not resp or resp["code"] != 0:
            await unsubscribe_uid(uid, groupid)
            BOT_Status.set_status(Status.SUBSCRIBE_IDLE, True)
            return f"UP（{uid}）订阅失败"
        if (
            int(dyn.extend.dyn_id_str) > BOT_Status.offset["$login"]
            if BOT_Status.offset["$login"]
            else 0
        ):
            BOT_Status.offset["$login"] = int(dyn.extend.dyn_id_str)
    elif not BotConfig.Bilibili.use_login:
        BOT_Status.offset[uid] = int(dyn.extend.dyn_id_str)

    if dyn.modules[0].module_author.author.live.live_state == 1:
        BOT_Status.living[uid] = 0
    BOT_Status.set_status(Status.SUBSCRIBE_IDLE, True)
    return f"成功在本群订阅 UP {up_name}（{uid}）"


async def unsubscribe_uid(uid, groupid, force=False):
    """在某个群退订某个 up"""
    uid = str(uid)
    groupid = str(groupid)
    logger.info(f"正在群 {groupid} 取消订阅 {uid}")
    BOT_Status.set_status(Status.SUBSCRIBE_IDLE, False)
    while (
        not BOT_Status.check_status(Status.DYNAMIC_IDLE)
        and not force
        and BotConfig.Bilibili.use_login
    ):
        await asyncio.sleep(0.1)

    if not uid_in_group(uid, groupid):
        BOT_Status.set_status(Status.SUBSCRIBE_IDLE, True)
        logger.info(f"群 {groupid} 未订阅 {uid}")
        return f"本群未订阅该 UP（{uid}）"
    up_name = get_sub_data(uid, groupid).uname
    if len(get_sub_by_uid(uid)) == 1:
        await delete_uid(uid)
    else:
        logger.info(f"正在从取消订阅 {uid}")
        unsub_uid_by_group(uid, groupid)
    BOT_Status.set_status(Status.SUBSCRIBE_IDLE, True)
    logger.info(f"成功从群 {groupid} 取消订阅 UP {up_name}（{uid}）")
    return f"{up_name}（{uid}）退订成功"


async def delete_uid(uid):
    """直接删除订阅的某个 up"""
    logger.info(f"正在从 BliBili 取消订阅 {uid}")
    uid = str(uid)
    if BotConfig.Bilibili.use_login:
        resp = await relation_modify(uid, 2)
        if resp and resp["code"] == 0:
            logger.info(f"取关 {uid} api 操作成功")
        else:
            logger.error(f"取关 {uid} 失败：{resp}")
            return False
    else:
        with contextlib.suppress(KeyError):
            del BOT_Status.living[uid]
        with contextlib.suppress(KeyError):
            del BOT_Status.offset[uid]
    if uid_exists(uid):
        delete_sub_by_uid(uid)
    else:
        logger.warning(f"{uid} 不存在于订阅列表中")
    logger.info(f"成功从 BliBili 取消订阅 {uid}")
    return True


def set_name(uid, name):
    uid = str(uid)

    if uid_exists(uid):
        set_uid_name(uid, name)
        return True
    else:
        return False


def set_nick(uid, groupid, nick):
    """设置某个 up 在某个群的昵称"""
    uid = str(uid)
    groupid = str(groupid)

    if uid_in_group(uid, groupid):
        data = get_sub_data(uid, groupid)
        data.nick = nick
        data.save()
        return True
    else:
        return False


def set_atall(uid, groupid, atall):
    """设置某个 up 在某个群的 @全体"""
    uid = str(uid)
    groupid = str(groupid)

    if uid_in_group(uid, groupid):
        data = get_sub_data(uid, groupid)
        data.atall = atall
        data.save()
        return True
    else:
        return False


async def delete_group(groupid) -> list[str]:
    """删除某个群的所有订阅"""
    remove_list = []
    logger.info(f"正在删除群 {groupid} 的所有订阅")
    for data in get_sub_by_group(groupid):
        await unsubscribe_uid(data.uid, data.group, True)
        remove_list.append(data.uid)
    return remove_list



import sys
import click
import httpx

from yarl import URL
from pathlib import Path
from json import JSONDecodeError
from noneprompt import (
    Choice,
    ListPrompt,
    InputPrompt,
    ConfirmPrompt,
    CancelledError,
    CheckboxPrompt,
)

from ..core import cache
from ..model.config import _BotConfig
from ..utils.detect_package import is_full
from ..core.announcement import BBOT_ASCII_LOGO


data = {}


class CliConfig:
    config: dict = {
        "Mirai": {},
        "Debug": {},
        "Bilibili": {},
        "Webui": {},
    }
    __session: str = ""

    def __init__(self) -> None:
        self.skip_verify = cache.get("skip_verify", False)
        self.mirai_mirai_host()
        self.mirai_verify_key()
        self.debug()
        self.use_browser()
        self.bilibili_concurrent()
        self.openai_summary()
        self.bilibili_username()
        self.use_bilibili_login()
        self.wordcloud()
        self.bcut_asr()
        self.event()
        self.webui()
        self.log_level()
        self.name()
        self.access_control()
        self.master()
        self.admins()
        self.max_subscriptions()
        self.update_check()

        if data.get("session"):
            httpx.post(
                f"{self.config['Mirai']['mirai_host']}/release",
                json={
                    "sessionKey": self.__session,
                    "qq": self.config["Mirai"]["account"],
                },
            )

    # ----- utils -----
    @staticmethod
    def is_qq(qq: str) -> bool:
        """判断是否为合法的 QQ 号"""
        return len(qq) >= 5 and len(qq) <= 17 if qq.isdigit() else False

    def verify_friend(self, qq: str) -> bool:
        """验证是否为好友"""
        friend_list = httpx.get(
            f"{self.config['Mirai']['mirai_host']}/friendList",
            params={"sessionKey": self.__session},
        ).json()["data"]
        return any(friend["id"] == int(qq) for friend in friend_list)

    def mirai_account(self):
        while True:
            mirai_account = InputPrompt("请输入你已经登录 Mirai 的 QQ 账号: ").prompt()
            if self.is_qq(mirai_account):
                self.config["Mirai"]["account"] = int(mirai_account)
                return
            click.secho("输入的 QQ 号不合法！", fg="bright_red", bold=True)

    # ----- Mirai -----
    def mirai_mirai_host(self):
        while True:
            mirai_host = InputPrompt(
                "请输入 Mirai HTTP API 的地址: ", "http://localhost:8080"
            ).prompt()
            if self.skip_verify:
                self.config["Mirai"]["mirai_host"] = mirai_host
                return
            try:
                if not URL(mirai_host).is_absolute():
                    raise ValueError("输入的地址不合法！")
            except Exception:
                click.secho("输入的地址不合法！", fg="bright_red", bold=True)
                continue

            try:
                if httpx.get(f"{mirai_host}/about").json()["data"]["version"] < "2.6.1":
                    click.secho(
                        "Mirai HTTP API 版本低于 2.6.1，可能会导致部分功能无法使用！请注意及时升级至最新版",
                        fg="bright_red",
                        bold=True,
                    )
                self.config["Mirai"]["mirai_host"] = mirai_host
                return
            except (KeyError, JSONDecodeError) as e:
                click.secho(
                    f"{type(e)}\n你输入的地址可能不是 Mirai HTTP API 的地址或 Mirai HTTP API 运行异常，请检查后重试！",
                    fg="bright_red",
                    bold=True,
                )
            except httpx.HTTPError:
                click.secho("无法连接到 Mirai HTTP API，请检查地址是否正确！", fg="bright_red", bold=True)
                continue

    def mirai_verify_key(self):
        while True:
            mirai_key: str = InputPrompt(
                "请输入 Mirai HTTP API 的 verifyKey: ", is_password=True
            ).prompt()
            if self.skip_verify:
                self.config["Mirai"]["verify_key"] = mirai_key
                return
            try:
                # check verifyKey
                verify = httpx.post(
                    f"{self.config['Mirai']['mirai_host']}/verify",
                    json={"verifyKey": mirai_key},
                ).json()
                if verify["code"] != 0:
                    click.secho("Mirai HTTP API 的 verifyKey 错误！", fg="bright_red", bold=True)
                    continue
                self.__session = verify["session"]

                # check bind
                while True:
                    self.mirai_account()
                    bind = httpx.post(
                        f"{self.config['Mirai']['mirai_host']}/bind",
                        json={
                            "sessionKey": self.__session,
                            "qq": self.config["Mirai"]["account"],
                        },
                    ).json()
                    if bind["code"] == 0 and (
                        httpx.get(
                            f"{self.config['Mirai']['mirai_host']}/sessionInfo",
                            params={"sessionKey": self.__session},
                        ).json()["code"]
                        == 0
                    ):
                        break
                    click.secho(
                        f"Mirai HTTP API 验证错误（可能是QQ号填写错误）：{bind['msg']}！",
                        fg="bright_red",
                        bold=True,
                    )

                # all clear
                click.secho("Mirai HTTP API 验证成功！", fg="bright_green", bold=True)
                self.config["Mirai"]["verify_key"] = mirai_key
                return

            except httpx.HTTPError:
                click.secho("无法连接到 Mirai HTTP API，请检查地址是否正确！", fg="bright_red", bold=True)
                self.mirai_mirai_host()
            except (KeyError, JSONDecodeError) as e:
                click.secho(
                    f"{type(e)}\n你输入的地址可能不是 Mirai HTTP API 的地址或 Mirai HTTP API 运行异常，请检查后重试！",
                    fg="bright_red",
                    bold=True,
                )
                self.mirai_mirai_host()

    def debug(self):
        debug = ListPrompt(
            "是否开启调试模式？（开启后 Bot 将只会响应调试群的消息）",
            [Choice("是（开启）"), Choice("否（关闭）")],
            allow_filter=False,
            default_select=1,
            annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
        ).prompt()
        if debug.name == "是（开启）":
            self.config["Debug"]["enable"] = True
            self.debug_group()
        else:
            self.config["Debug"] = {"enable": False, "groups": [123456789]}

    def debug_group(self):
        self.config["Debug"]["groups"] = []
        while True:
            debug_group = InputPrompt("请输入调试群的群号（输入 n 停止）: ").prompt()
            if debug_group.lower() == "n":
                if not self.config["Debug"]["groups"]:
                    click.secho("请至少输入一个群号！", fg="bright_red", bold=True)
                    continue
                return
            if not debug_group.isdigit():
                click.secho("输入的群号不合法！", fg="bright_red", bold=True)
                continue
            if int(debug_group) in self.config["Debug"]["groups"]:
                click.secho("该群已在调试群列表中，请重新输入！", fg="bright_red", bold=True)
                continue
            debug_group = int(debug_group)
            if self.skip_verify:
                self.config["Debug"]["groups"].append(debug_group)
                return
            group_list = httpx.get(
                f"{self.config['Mirai']['mirai_host']}/groupList",
                params={"sessionKey": self.__session},
            ).json()["data"]

            for group in group_list:
                if group["id"] == debug_group:
                    self.config["Debug"]["groups"].append(debug_group)
                    break
            else:
                click.secho("Bot 未加入该群，请重新输入！", fg="bright_red", bold=True)

    def use_browser(self):
        if is_full:
            browser = ListPrompt(
                "是否使用浏览器进行动态页面截图？",
                [Choice("是（开启）"), Choice("否（关闭）")],
                allow_filter=False,
                default_select=1,
                annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
            ).prompt()
            self.config["Bilibili"]["use_browser"] = browser.name == "是（开启）"
            self.bilibili_mobile_style()
            self.render_style()
            self.allow_fallback()
        else:
            self.config["Bilibili"]["use_browser"] = False

    def bilibili_mobile_style(self):
        if is_full:
            mobile_style = ListPrompt(
                "是否在浏览器中使用手机端样式？",
                [Choice("是（开启）"), Choice("否（关闭）")],
                allow_filter=False,
                annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
            ).prompt()
            self.config["Bilibili"]["mobile_style"] = mobile_style.name == "是（开启）"
        else:
            self.config["Bilibili"]["mobile_style"] = False

    def render_style(self):
        if is_full:
            mobile_style = ListPrompt(
                "请选择使用的视频信息渲染模板？（bbot_default无需浏览器，其他均需求浏览器）",
                [Choice("bbot_default"), Choice("style_blue")],
                allow_filter=False,
                annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
            ).prompt()
            self.config["Bilibili"]["render_style"] = mobile_style.name
        else:
            self.config["Bilibili"]["render_style"] = "bbot_default"

    def allow_fallback(self):
        allow_fallback = ListPrompt(
            "是否允许使用备用动态图片渲染（在浏览器截图失败时尝试使用）？",
            [Choice("是（开启）"), Choice("否（关闭）")],
            allow_filter=False,
            annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
        ).prompt()
        self.config["Bilibili"]["allow_fallback"] = allow_fallback.name == "是（开启）"

    def openai_summary(self):
        openai_summary = ListPrompt(
            "是否使用 OpenAI 进行视频和专栏内容摘要提取？",
            [Choice("是（开启）"), Choice("否（关闭）")],
            allow_filter=False,
            annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
        ).prompt()
        if openai_summary.name == "是（开启）":
            self.config["Bilibili"]["openai_summarization"] = True
            self.openai_api_token()
            self.openai_model()
        else:
            self.config["Bilibili"]["openai_summarization"] = False

    def openai_api_token(self):
        openai_token = InputPrompt("请输入 OpenAI Token: ").prompt()
        if len(openai_token) != 51:
            click.secho("输入的 OpenAI Token 不合法（长度应为 51 位）", fg="bright_red", bold=True)
            self.openai_api_token()
        if openai_token.startswith("sk-"):
            self.config["Bilibili"]["openai_api_token"] = openai_token
        else:
            click.secho("输入的 OpenAI Token 不合法（应以 sk- 开头）", fg="bright_red", bold=True)
            self.openai_api_token()

    def openai_model(self):
        openai_model = ListPrompt(
            "请选择 OpenAI 模型",
            [
                Choice("gpt-3.5-turbo-0613"),
                Choice("gpt-3.5-turbo-16k-0613"),
                Choice("gpt-4-0613"),
            ],
            allow_filter=False,
            annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
        ).prompt()
        self.config["Bilibili"]["openai_model"] = openai_model.name

    def openai_proxy(self):
        if openai_proxy_url := InputPrompt("请输入 OpenAI 代理地址（留空则不使用代理）: ").prompt():
            if not URL(openai_proxy_url).is_absolute():
                click.secho("输入的 OpenAI 代理地址不合法！", fg="bright_red", bold=True)
                self.openai_proxy()
            elif not openai_proxy_url.startswith("http"):
                click.secho("输入的 OpenAI 代理地址不合法！仅可使用 http 代理", fg="bright_red", bold=True)
                self.openai_proxy()
            self.config["Bilibili"]["openai_proxy"] = openai_proxy_url

    def bilibili_username(self):
        username = InputPrompt("请输入 Bilibili 用户名: （可用于 AI 总结时获取 Bilibili 的 AI 字幕）").prompt()
        if not username or username == "":
            self.config["Bilibili"]["username"] = username
            click.secho("用户名为空，已关闭对应功能！", fg="bright_red", bold=True)
            return 
        elif not username.isdigit():
            click.secho("用户名不合法！", fg="bright_red", bold=True)
            return self.bilibili_username()
        self.config["Bilibili"]["username"] = username
        self.bilibili_password()

    def bilibili_password(self):
        password = InputPrompt("请输入 Bilibili 密码: ", is_password=True).prompt()
        if not password:
            click.secho("密码不能为空！", fg="bright_red", bold=True)
            self.bilibili_password()
        self.config["Bilibili"]["password"] = password

    def use_bilibili_login(self):
        if self.config["Bilibili"]["username"]:
            use_bilibili_login = ListPrompt(
                "是否使用已登录的 Bilibili 账号进行动态监听？（警告：不推荐使用该功能，暂不可用）",
                [Choice("否（关闭）"), Choice("是（开启）")],
                allow_filter=False,
                annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
            ).prompt()
            self.config["Bilibili"]["use_login"] = use_bilibili_login.name == "是（开启）"
        else:
            self.config["Bilibili"]["use_login"] = False

    def wordcloud(self):
        if is_full:
            wordcloud = ListPrompt(
                "是否为视频和专栏内容制作词云？",
                [Choice("是（开启）"), Choice("否（关闭）")],
                allow_filter=False,
                annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
            ).prompt()
            self.config["Bilibili"]["use_wordcloud"] = wordcloud.name == "是（开启）"
        else:
            self.config["Bilibili"]["use_wordcloud"] = False

    def bcut_asr(self):
        if (
            self.config["Bilibili"]["openai_summarization"]
            or self.config["Bilibili"]["use_wordcloud"]
        ):
            bcut_asr = ListPrompt(
                "是否使用 Bilibili ASR 进行视频内容语音识别？（用于 AI 总结和词云制作）",
                [Choice("是（开启）"), Choice("否（关闭）")],
                allow_filter=False,
                annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
            ).prompt()
            self.config["Bilibili"]["use_bcut_asr"] = bcut_asr.name == "是（开启）"
        else:
            self.config["Bilibili"]["use_bcut_asr"] = False

    def bilibili_concurrent(self):
        while True:
            concurrent = InputPrompt("请输入并发数（理论该值越大推送效率越高）: ", default_text="10").prompt()
            if concurrent.isdigit() and 50 >= int(concurrent) > 0:
                self.config["Bilibili"]["concurrency"] = int(concurrent)
                return
            click.secho("输入的并发数不合法！请输入 1 - 50 之间的整数", fg="bright_red", bold=True)

    def event(self):
        event_map = {"禁言": "mute", "权限变更": "permission", "动态推送": "push", "UP 订阅": "subscribe"}
        event_list = CheckboxPrompt(
            "请选择需要私聊管理员推送的事件",
            [Choice("禁言"), Choice("权限变更"), Choice("动态推送"), Choice("UP 订阅")],
            default_select=[0, 1, 2, 3],
            annotation="使用键盘的 ↑ 和 ↓ 来选择, 按空格开关, 按回车确认",
        ).prompt()
        self.config["Event"] = {event_map[event.name]: True for event in event_list}

    def webui(self):
        webui = ListPrompt(
            "是否开启 WebUI？",
            [Choice("是（开启）"), Choice("否（关闭）")],
            allow_filter=False,
            annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
        ).prompt()
        if webui.name == "是（开启）":
            self.webui_data()
        else:
            self.config["Webui"] = {
                "webui_host": "0.0.0.0",
                "webui_port": 6080,
                "webui_enable": False,
            }

    def webui_data(self):
        while True:
            webui_url = InputPrompt(
                "请输入 WebUI 的地址: ", default_text="http://0.0.0.0:6080"
            ).prompt()
            try:
                url = URL(webui_url)
                if not url.is_absolute():
                    raise ValueError("输入的地址不合法！")
            except Exception:
                click.secho("输入的地址不合法！", fg="bright_red", bold=True)
                continue
            self.config["Webui"] = {
                "webui_host": url.host,
                "webui_port": url.port,
                "webui_enable": True,
            }
            return

    # ----- Other -----
    def master(self):
        while True:
            master = InputPrompt("请输入你的 QQ 号（作为最高管理员账号）: ").prompt()
            if not self.is_qq(master):
                click.secho("输入的 QQ 号不合法！", fg="bright_red", bold=True)
                continue

            if self.skip_verify or self.verify_friend(master):
                self.config["master"] = int(master)
                return
            else:
                click.secho("你输入的 QQ 号不是 Bot 的好友！", fg="bright_red", bold=True)

    def admins(self):
        self.config["admins"] = [self.config["master"]]
        while True:
            admin = InputPrompt("请输入其他管理员的 QQ 号（输入 n 停止）: ").prompt()
            if admin.lower() == "n" or admin == "":
                return
            if not self.is_qq(admin):
                click.secho("输入的 QQ 号不合法！", fg="bright_red", bold=True)
                continue
            if int(admin) in self.config["admins"]:
                click.secho("该 QQ 号已经在管理员列表中了！", fg="bright_red", bold=True)
                continue
            if not self.skip_verify and not self.verify_friend(admin):
                click.secho("该 QQ 号不是 Bot 的好友！", fg="bright_red", bold=True)
                continue

            self.config["admins"].append(int(admin))

    def log_level(self):
        self.config["log_level"] = (
            ListPrompt(
                "请选择日志等级",
                [
                    Choice("INFO"),
                    Choice("DEBUG"),
                    Choice("WARNING"),
                ],
                allow_filter=False,
                annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
            )
            .prompt()
            .name
        )

    def name(self):
        while True:
            bot_name = InputPrompt("请输入 Bot 的昵称: ", default_text="BBot").prompt()
            if 0 < len(bot_name) <= 16:
                self.config["name"] = bot_name
                return
            click.secho("输入的昵称不合法（请输入 1 - 16 位字符）", fg="bright_red", bold=True)

    def max_subscriptions(self):
        while True:
            max_subscriptions = InputPrompt("请输入每个群的最大订阅数: ", default_text="4").prompt()
            if max_subscriptions.isdigit() and 0 < int(max_subscriptions) <= 100:
                self.config["max_subsubscribe"] = int(max_subscriptions)
                return
            click.secho("输入的订阅数不合法！请输入 1 - 100 之间的整数", fg="bright_red", bold=True)

    def access_control(self):
        access_control = ListPrompt(
            "是否开启白名单控制？（仅允许加入白名单中的群）",
            [Choice("是（开启）"), Choice("否（关闭）")],
            allow_filter=False,
            annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
        ).prompt()
        self.config["access_control"] = access_control.name == "是（开启）"

    def update_check(self):
        update_check = ListPrompt(
            "是否开启更新检查？",
            [Choice("是（开启）"), Choice("否（关闭）")],
            allow_filter=False,
            annotation="使用键盘的 ↑ 和 ↓ 来选择, 按回车确认",
        ).prompt()
        self.config["update_check"] = update_check.name == "是（开启）"


def click_config(ignore_exist=False):
    click.secho(BBOT_ASCII_LOGO, fg="bright_blue", bold=True)
    click.secho("\n欢迎使用 BBot 配置向导！\n", fg="bright_green", bold=True)
    click.secho("请按照提示输入相关信息后，按回车确认\n", fg="bright_yellow")

    try:
        _config(ignore_exist)
        sys.exit(0)
    except CancelledError:
        click.secho("配置向导已取消。", fg="bright_yellow", bold=True)
        sys.exit(0)


def _config(ignore_exist):
    if list(Path(".").iterdir()) and not ignore_exist:
        click.secho("当前目录不为空，可能会导致未知的错误。", fg="bright_yellow", bold=True)
        click.secho("请确保当前目录没有 Bot 之外的文件。", fg="bright_yellow", bold=True)

        if not ConfirmPrompt("是否仍要继续配置流程？", default_choice=False).prompt():
            sys.exit(0)
        if Path("data", "bot_config.yaml").exists():
            click.secho("检测到已存在的配置文件，将覆盖原配置。", fg="bright_yellow", bold=True)
            if not ConfirmPrompt("是否仍要继续配置流程？", default_choice=False).prompt():
                sys.exit(0)

    if not Path("data").exists():
        click.secho("data 目录不存在，已自动创建", fg="bright_yellow")
        Path("data").mkdir(parents=True, exist_ok=True)

    click.secho("正在进入配置流程……", fg="bright_green", bold=True)
    _BotConfig.parse_obj(CliConfig().config).save()

    click.secho("恭喜你，配置已完成！", fg="bright_green", bold=True)
    click.secho("现在，你可以使用 bbot run 命令运行 BBot 了。", fg="bright_blue", bold=True)
